################################################################################
#
#   Gene prediction pipeline 
#
#   $Id: gtf2gff.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
gtf2gff.py - convert a transcript set to genomic features
=========================================================

:Author: Andreas Heger
:Release: $Id: gtf2gff.py 2861 2010-02-23 17:36:32Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

This scripts converts a transcript set in a :term:`gtf` formatted file into a set
of features in a :term:`gff` formatted file. 

In other words, a gene set (gtf), which constitutes a hierarchical set of 
annotations, will be converted into a non-hierarchical list of genomic segments.

Various methods can be used to do the conversion (see command line argument 
``--method``):

exons
   annotate exons. Exonic segments are classified according 
   to the transcript structure.

genome/full
   annotate genome with gene set. Genomic segments are labeled 
   ``intronic``, ``intergenic``, etc. This annotation aggregates
   the information of multiple genes such that each annotation
   is either valid or ambiguous.

genes
    annotate genome using the information on a gene-by-gene basis.
    Multiple overlapping annotations will be created for each transcript.
    Redundant annotations will be merged.

great-domains
   regulatory domains using the basal+extended model according to GREAT.

promotors
   declare promoter regions. These segments might be overlapping. A promotor
   is the region x kb upstream of a transcription start site. The option
   ``--promotor`` sets the region width.

regulons
   declare regulatory regions. Regulatory regions contain the region x kb of
   upstream and downstream of a transciption start site. The options ``--upstream``
   and ``-downstream`` set the region width.

tts-regulons
   declare tts regulatory regions. tts-regulatory regions contain the region x kb of
   upstream and downstream of a transciption termination site. The options ``--upstream``
   and ``-downstream`` set the region width.

territories
   build gene territories around full length genes.

tss-territories
   build gene territories around transcription start sites.


Genome
++++++

If ``--method=genome``, the gene set is used to annotate the complete genome.

.. note::
   The gtf file has to be sorted first by contig and then by position.

A segment in the genome will either be covered by:

cds
   a coding exon (also: CDS, start_codon).

utr
   a UTR (also: stop_codon) 

5flank, 3flank, flank
   an upstream/downstream segment of defined size. If 
   the intergenic region is too small to accomodate a flank, the regions is
   just 'flank'.
                
intergenic
   intergenic region.

5telomeric, 3telomeric
   telomeric region (before/after first/last gene).

intronic
   intronic region. An intron has a minimum size of 30 bases.

frameshift
   frameshift. Introns of less than 4 residues length

ambiguous
   in case of overlapping genes, regions are designated ambiguous

unknown
   unknown are ``intronic`` regions that are less than the minimum size 
   of an intron (default: 30) and larger than the size of frameshift (default:4).
   These could be either genuine small introns or they could be artefactual arising
   from collapsing the exons within a gene model.

All segments are annotated by their closest gene. Intergenic regions are
annotated with their two neighbouring genes. The upstream gene is listed
in the attribute gene_id, the downstream one is listed in the attribute
downstream_gene_id.

Genes
+++++

If ``--method=genes``, the gene set is used to annotate the complete genome.

.. note::
   The gtf file has to be sorted by gene.

A segment in the genome will be annotated as:

cds
   a coding exon

utr5, utr3
   a 5' or 3' utr

exon
   an exon. Exons are further classified into first, middle and last exons.

intronic
   an intronic region. Intronic regions are further divided into first, middle, last.
   
upstream, downstream
   upstream/downstream regions in 5 intervals of a total of 1kb (see option --flank to increase
   the total size).

.. _territories:

Territories
+++++++++++

If ``--method=territories``, the gene set is used to define gene territories. 
Territories are segments around genes and are non-overlapping. Exons in a gene
are merged and the resulting the region is enlarged by --radius. Overlapping
territories are divided at the midpoint between the two genes. The maximum
extent of a territory is limited by the option ``--radius``

.. note::
   The gtf file has to be sorted first by contig and then by position.

.. note::
   Genes should already have been merged (gtf2gtf --merge-transcripts)

TSSTerritories
++++++++++++++

If ``--method=tss-territories``, the gene set is used to define gene territories. 
Instead of the full gene length as in :ref:`territories`, only the tss is used to 
define a territory. Territories are segments around genes and are non-overlapping.
Overlapping territories are divided at the midpoint between the two genes. The maximum
extent of a territory is limited by the option ``--radius``.

.. note::
   The gtf file has to be sorted first by contig and then by position.

.. note::
   Genes should already have been merged (gtf2gtf --merge-transcripts)

The domain definitions corresponds to the ``nearest gene`` rule in GREAT.

GREAT-Domains
+++++++++++++

Define GREAT regulatory domains. Each TSS in a gene is associated with a 
basal region. The basal region is then extended upstream to the basal region 
of the closest gene, but at most to --radius. In the case of overlapping genes,
the extension is towards the next non-overlapping gene. 

This is the "basal plus extension" rule in GERAT. Commonly used are 5+1 with 1 Mb extension. 
To achieve this, set

``--method=great-domains --upstream=5000 --downstream=1000 --radius=1000000``

If there are a multiple TSS in a transcript, the basal region extends from the
first to the last TSS plus the upstream/downstream flank.

Exons
+++++

If ``--method=exons``, exons are annotated by their dispensibility.

.. note::
   The gtf file should be sorted by genes

For each exon, the following additional fields are added to the gtf file:

ntranscripts
   number of transcripts
nused
   number of transcripts using this exon
positions
   positions of exon within transcripts. This is a ``,`` separated list of
   tuples ``pos:total``. For example, ``1:10,5:8`` indicates an exon that appears
   in first position in a ten exon transcript and fifth position in an eight
   exon transcript. The position is according to the direction of transcription.

.. note::
   overlapping but non-identical exons, for example due to internal splice sites, are listed 
   as separate exons. Thus the output is not fully flat as some segments could be overlapping
   (see output variable ``noverlapping`` in the log file).

The following example uses an ENSEMBL gene set::

   gunzip < Mus_musculus.NCBIM37.55.gtf.gz | awk '$3 == "CDS"' | python gtf2gff.py --method=exons --restrict-source=protein_coding

Promoters
+++++++++

If ``--method=promotors``, putative promotor regions are output. A
promoter is a pre-defined segment upstream of the transcription start
site. As the actual start site is usually not known, the start of the
first exon within a transcript is used as a proxy. A gene can have
several promotors associated with it, but overlapping promotor regions
of the same gene will be merged. A promoter can extend into an
adjacent upstream gene.

The ``--restrict-source`` option determines which GTF entries are output. The default
is to output all entries but the user can choose from protein_coding, pseudogene or lncRNA.

The size of the promotor region can be specified by the command line
argument ``--promotor``.

Regulons
+++++++++

If ``--method=regulons``, putative regulon regions are output. This is similar
to a ``promotor``, but the region extends both upstream and downstream from
the transcription start site.

The ``--restrict-source`` option determines which GTF entries are output. The default
is to output all entries but the user can choose from protein_coding, pseudogene or lncRNA.

The size of the promotor region can be specified by the command line
argument ``--upstream`` and ``--downstream``

If ``--method=tts-regulons``, regulons will be defined around the transcription
termination site.
 
Usage
-----

Type::
    python gtf2gff.py --method=genome --genome-file=hg19 < geneset.gtf > annotations.gff

for command line help.

Command line options
---------------------

"""

import os
import sys
import string
import re
import types
import collections
import itertools

import CGAT.GTF as GTF
import CGAT.Experiment as E
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Genomics as Genomics
import CGAT.Intervals as Intervals
import CGAT.IOTools as IOTools

def addSegment( feature, start, end, template, options ):
    """add a generic segment of type *feature*.
    """
    if start >= end: return 0

    entry = GTF.Entry()

    if type(template) == types.TupleType:
        entry.copy( template[0] )        
        entry.clearAttributes()
        entry.addAttribute( "downstream_gene_id", template[1].gene_id )
    else:
        entry.copy( template )
        entry.clearAttributes()

    entry.start, entry.end = start, end
    entry.feature = feature
    if feature not in ("exon", "CDS", "UTR", "UTR3", "UTR5"):
        entry.score = "."
    options.stdout.write( str(entry) + "\n" )

    return 1

def addFlank( start, end, template, options ):
    """add a flank.
    """    
    is_positive = Genomics.IsPositiveStrand( template.strand )
    is_before = end <= template.start
    if (is_before and is_positive) or (not is_before and not is_positive):
        name = "5flank"
    else:
        name = "3flank"
            
    return addSegment( name, start, end, template, options )

def addIntergenicSegment( last, this, fasta, options ):
    """add an intergenic segment between last and this.

    At telomeres, either can be None.
    """
    if not this and not last: return 0
        
    nadded = 0
    if not this:
        # last telomere
        try:
            lcontig = fasta.getLength( last.contig )
        except KeyError, msg:
            if options.ignore_missing:
                return nadded
            else:
                raise KeyError, msg
        flank = min( last.end + options.flank, lcontig )
        nadded += addFlank( last.end, flank, last, options )
        nadded += addSegment( "telomeric", flank, lcontig, last, options )
    elif not last:
        # first telomere
        flank = max(0, this.start - options.flank )
        nadded += addSegment( "telomeric", 0, flank, this, options )
        nadded += addFlank( flank, this.start, this, options )
    else:
        # intergenic region
        d = this.start - last.end
        flank = options.flank
        if d > flank * 2:
            nadded += addFlank( last.end, last.end + flank, last, options )
            nadded += addSegment( "intergenic", last.end + flank, this.start - flank, (last,this), options )
            nadded += addFlank( this.start - flank, this.start, this, options )
        else:
            # add short flank between two genes. If they can not agree
            # on the directionality, "flank" is used.
            is_positive1 = Genomics.IsPositiveStrand( last.strand ) 
            is_positive2 = Genomics.IsPositiveStrand( this.strand )
            if is_positive1 and not is_positive2:
                key = "3flank"
            elif not is_positive1 and is_positive2:
                key = "5flank"
            else:
                key = "flank"
            nadded += addSegment( key, last.end, this.start, (last,this), options )

    return nadded

##-----------------------------------------------------------------------------
def buildTerritories( iterator, fasta, method, options ):
    """build gene territories. 

    Exons in a gene are merged and the resulting 
    segments enlarged by --radius. Territories
    overlapping are divided in the midpoint between
    the two genes.

    If *method* is ``gene``, gene territories will be built.
    If *method* is ``tss``, tss territories will be built.

    """

    ninput, noutput, nambiguous = 0, 0, 0

    assert method in ("gene", "tss")

    dr = 2 * options.radius

    prev_pos = 0
    last_contig = None
    gff = None

    def _iterator( iterator ):
        """yield gene plus the locations of the end of the previous gene and start of next gene"""

        last_end, prev_end = 0, 0
        last_contig = None
        last = None
        for matches in GTF.iterator_overlaps( iterator ):

            this_start = min( [ x.start for x in matches ] )
            this_end = max( [ x.end for x in matches ] )

            if method == "tss":
                # restrict to tss 
                if matches[0].strand == "+":
                    this_end = this_start + 1
                else:
                    this_start = this_end - 1

            this_contig = matches[0].contig

            if last_contig != this_contig: 
                if last:
                    yield prev_end, last, fasta.getLength( last_contig )
                last_end, prev_end = 0, 0
            else:
                yield prev_end, last, this_start

            prev_end = last_end
            last_end = this_end
            last = matches
            last_contig = this_contig
            
        if last:
            yield prev_end, last, fasta.getLength( last_contig )

    for last_end, matches, next_start in _iterator( iterator ):

        gff = GTF.Entry().copy( matches[0] )

        start = min( [ x.start for x in matches ] )
        end = max( [ x.end for x in matches ] )

        if method == "tss":
            # restrict to tss 
            if matches[0].strand == "+":
                end = start + 1
            else:
                start = end - 1

        d = start - last_end
        if d < dr:
            start -= d // 2
        else:
            start -= options.radius

        d = next_start - end
        if d < dr:
            end += d // 2
        else:
            end += options.radius

        gff.gene_id = ":".join( set( [x.gene_id for x in matches ] ) )
        gff.transcript_id = gff.gene_id
        gff.start, gff.end = start, end

        nsegments = len(matches)
        if nsegments > 1: 
            gff.addAttribute("ambiguous", nsegments )
            nambiguous += 1

        assert gff.start < gff.end, "invalid segment: %s" % str(gff)
        options.stdout.write( str(gff) + "\n" )
        noutput += 1

    E.info( "ninput=%i, noutput=%i, nambiguous=%i" % (ninput, noutput, nambiguous ))

##-----------------------------------------------------------------------------
def annotateGenome( iterator, fasta, options ):
    """perform a full segmentation of the genome (UTR, exon, intron ...)
    """

    ninput, noutput, nadded, nambiguous, nframeshifts, nunknown = 0, 0, 0, 0, 0, 0
    last = None
    is_ambiguous = False

    for this in iterator:
        ninput += 1

        E.debug( "last=%s" % str(last))
        E.debug( "this=%s" % str(this))
        E.debug( "is_ambiguous=%s" % str(is_ambiguous))

        if last and last.contig == this.contig:
            # check if file is sorted correctly
            assert last.start <= this.start, "input file needs to be sorted by contig, start" 
            if last.end <= this.start:
                if not is_ambiguous:
                    if last.gene_id != this.gene_id:
                        nadded += addIntergenicSegment( last, this, fasta, options )
                    else:
                        d = this.start - last.end  
                        if d >= options.min_intron_length:
                            nadded += addSegment( "intronic", last.end, this.start, last, options )
                        elif d <= options.max_frameshift_length:
                            nframeshifts += addSegment( "frameshift", last.end, this.start, last, options )
                        else:
                            nunknown += addSegment( "unknown", last.end, this.start, last, options )
                else:
                    if last.feature == this.feature and last.gene_id == this.gene_id:
                        nambiguous += addSegment( last.feature, last.end, this.start, last, options )
                    else:
                        nambiguous += addSegment( "ambiguous", last.end, this.start, last, options )
                    is_ambiguous = False
                last = this
            elif last.end > this.start:
                if last.gene_id != this.gene_id:
                    # flag next region as ambiguous
                    is_ambiguous = True
                last.end = this.end
        else:
            nadded += addIntergenicSegment( last, None, fasta, options )
            nadded += addIntergenicSegment( None, this, fasta, options )
            last = this

        options.stdout.write( "%s\n" % str(this) )
        noutput += 1
            
    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nadded=%i, nambiguous=%i, nframeshifts=%i, nunknown=%i\n" % (ninput, noutput, nadded, nambiguous, nframeshifts, nunknown) )

def annotateExons( iterator, fasta, options ):
    """annotate exons within iterator."""

    gene_iterator = GTF.gene_iterator( iterator )

    ninput, noutput, noverlapping = 0, 0, 0

    for this in gene_iterator:
        ninput += 1
        intervals = collections.defaultdict( list )
        ntranscripts = len(this)

        is_negative_strand = Genomics.IsNegativeStrand( this[0][0].strand )

        for exons in this:
            # make sure these are sorted correctly
            exons.sort( key=lambda x: x.start )
            if is_negative_strand: exons.reverse()

            nexons = len(exons)
            for i, e in enumerate(exons):
                intervals[ (e.start, e.end) ].append( (i+1,nexons) )

        gtf = GTF.Entry()
        gtf.fromGTF( this[0][0], this[0][0].gene_id, this[0][0].gene_id )
        gtf.addAttribute( "ntranscripts", ntranscripts )        

        gtfs = []
        for r, pos in intervals.iteritems():

            g = GTF.Entry().copy(gtf)
            g.start, g.end = r
            g.addAttribute( "nused", len(pos) )
            g.addAttribute( "pos", ",".join( ["%i:%i" % x for x in pos ] ) )
            gtfs.append( g )
        
        gtfs.sort( key=lambda x: x.start )
        
        for g in gtfs:
            options.stdout.write( "%s\n" % str(g) )

        # check for exon overlap    
        intervals = [ (g.start, g.end) for g in gtfs ]
        nbefore = len(intervals)
        nafter = len( Intervals.combine( intervals ) )
        if nafter != nbefore: noverlapping += 1

        noutput += 1
            
    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, noverlapping=%i\n" % (ninput, noutput, noverlapping) )

def annotatePromoters( iterator, fasta, options ):
    """annotate promoters within iterator.

    Entries specied with ``--restrict-source`` are annotated.
    """

    gene_iterator = GTF.gene_iterator( iterator )

    ngenes, ntranscripts, npromotors = 0, 0, 0

    for gene in gene_iterator:
        ngenes += 1
        is_negative_strand = Genomics.IsNegativeStrand( gene[0][0].strand )
        lcontig = fasta.getLength( gene[0][0].contig )
        promotors = []
        transcript_ids = []
        for transcript in gene:

            ntranscripts += 1
            mi, ma = min( [x.start for x in transcript ] ), max( [x.end for x in transcript ] )
            transcript_ids.append( transcript[0].transcript_id )
            # if tss is directly at start/end of contig, the tss will be within an exon.
            # otherwise, it is outside an exon.
            if is_negative_strand:
                promotors.append( (min( lcontig-options.promotor, ma), min(lcontig, ma + options.promotor)) )
            else:
                promotors.append( (max(0,mi - options.promotor), max(options.promotor,mi)) )

        if options.merge_promotors:
            # merge the promotors (and rename - as sort order might have changed)
            promotors = Intervals.combine( promotors )
            transcript_ids = ["%i" % (x+1) for x in range(len(promotors) )]
            
        gtf = GTF.Entry()
        gtf.fromGTF( gene[0][0], gene[0][0].gene_id, gene[0][0].gene_id )
        gtf.source = "promotor"

        x = 0
        for start, end in promotors:
            gtf.start, gtf.end = start, end
            gtf.transcript_id = transcript_ids[x]
            options.stdout.write( "%s\n" % str(gtf) )
            npromotors += 1
            x += 1

    E.info( "ngenes=%i, ntranscripts=%i, npromotors=%i" % (ngenes, ntranscripts, npromotors) )

def annotateRegulons( iterator, fasta, tss, options ):
    """annotate regulons within iterator.

    Entries specied with ``--restrict-source`` are annotated.
    """

    gene_iterator = GTF.gene_iterator( iterator )

    ngenes, ntranscripts, nregulons = 0, 0, 0

    upstream, downstream = options.upstream, options.downstream

    for gene in gene_iterator:
        ngenes += 1
        is_negative_strand = Genomics.IsNegativeStrand( gene[0][0].strand )
        lcontig = fasta.getLength( gene[0][0].contig )
        regulons = []
        transcript_ids = []
        for transcript in gene:

            ntranscripts += 1
            mi, ma = min( [x.start for x in transcript ] ), max( [x.end for x in transcript ] )
            if tss:
                # add range to both sides of tss
                if is_negative_strand:
                    interval = ma - options.downstream, ma + options.upstream
                else:
                    interval = mi - options.upstream, mi + options.downstream
            else:
                # add range to both sides of tts
                if is_negative_strand:
                    interval = mi - options.downstream, mi + options.upstream
                else:
                    interval = ma - options.upstream, ma + options.downstream

            interval = ( min( lcontig, max( 0, interval[0] ) ),
                         min( lcontig, max( 0, interval[1] ) ) )
            
            regulons.append( interval )
            transcript_ids.append( transcript[0].transcript_id )

        if options.merge_promotors:
            # merge the regulons (and rename - as sort order might have changed)
            regulons = Intervals.combine( regulons )
            transcript_ids = ["%i" % (x+1) for x in range(len(regulons) )]
            
        gtf = GTF.Entry()
        gtf.fromGTF( gene[0][0], gene[0][0].gene_id, gene[0][0].gene_id )
        gtf.source = "regulon"

        x = 0
        for start, end in regulons:
            gtf.start, gtf.end = start, end
            gtf.transcript_id = transcript_ids[x]
            options.stdout.write( "%s\n" % str(gtf) )
            nregulons += 1
            x += 1

    E.info( "ngenes=%i, ntranscripts=%i, nregulons=%i" % (ngenes, ntranscripts, nregulons) )

####################################################################
####################################################################
####################################################################
def annotateGREATDomains( iterator, fasta, options ):
    """build great domains

    extend from TSS a basal region.

    """

    gene_iterator = GTF.gene_iterator( iterator )

    counter = E.Counter()

    upstream, downstream = options.upstream, options.downstream
    radius = options.radius
    outfile = options.stdout

    regions = []
    ####################################################################
    # define basal regions for each gene
    # take all basal regions per transcript and merge them
    # Thus, the basal region of a gene might be larger than the sum
    # of options.upstream + options.downstream
    for gene in gene_iterator:
        counter.genes += 1
        is_negative_strand = Genomics.IsNegativeStrand( gene[0][0].strand )

        lcontig = fasta.getLength( gene[0][0].contig )
        regulons = []
        transcript_ids = []

        # collect every basal region per transcript
        for transcript in gene:
            counter.transcripts += 1
            mi, ma = min( [x.start for x in transcript ] ), max( [x.end for x in transcript ] )
            # add range to both sides of tss
            if is_negative_strand:
                interval = ma - options.downstream, ma + options.upstream
            else:
                interval = mi - options.upstream, mi + options.downstream

            interval = ( min( lcontig, max( 0, interval[0] ) ),
                         min( lcontig, max( 0, interval[1] ) ) )
            
            regulons.append( interval )
            transcript_ids.append( transcript[0].transcript_id )

        # take first/last entry 
        start, end = min( x[0] for x in regulons), max(x[1] for x in regulons )
        
        gtf = GTF.Entry()
        gtf.fromGTF( gene[0][0], gene[0][0].gene_id, gene[0][0].gene_id )
        gtf.source = "greatdomain"
        gtf.start, gtf.end = start, end
        regions.append( gtf )

    regions.sort( key = lambda x: (x.contig, x.start ) )

    outf = IOTools.openFile( "test.gff", "w")
    for x in regions:
        outf.write( str(x) + "\n" )
    outf.close()

    ####################################################################
    # extend basal regions
    regions.sort( key = lambda x: (x.contig, x.start ) )

    # iterate within groups of overlapping basal regions
    groups = list( GTF.iterator_overlaps( iter(regions) ))
    counter.groups = len(groups)

    last_end = 0
    reset = False

    for region_id, group in enumerate(groups):

        # collect basal intervals in group
        intervals = [ (x.start, x.end) for x in group ]

        def overlapsBasalRegion( pos ):
            for start,end in intervals:
                if start == pos or end == pos: continue
                if start <= pos < end:
                    return True
                if start > pos: return False
            return False

        # deal with boundary cases - end of contig
        if region_id < len(groups) - 1:
            nxt = groups[region_id+1]
            if nxt[0].contig == group[0].contig:
                next_start = min( [x.start for x in nxt] )
            else:
                next_start = fasta.getLength(group[0].contig)
                reset = True
        else:
            next_start = fasta.getLength(group[0].contig)
            reset = True

        # last_end = basal extension of previous group
        # next_start = basal_extension of next group

        # extend region to previous/next group
        # always extend dowstream, but upstream only extend if basal region of an interval is not already 
        # overlapping another basal region within the group
        save_end = 0
        for gtf in group:
            save_end = max(save_end, gtf.end)
            if gtf.strand == "+":
                if not overlapsBasalRegion( gtf.start ):
                    gtf.start = max( gtf.start - radius, last_end )
                # always extend downstream    
                gtf.end = min( gtf.end + radius, next_start )
            else:
                # always extend downstream
                gtf.start = max( gtf.start - radius, last_end )
                if not overlapsBasalRegion( gtf.end ):
                    gtf.end = min( gtf.end + radius, next_start )
            outfile.write( str(gtf) + "\n" )
            counter.regulons += 1

        if len(group) > 1: 
            counter.overlaps += len(group)
        else:
            counter.nonoverlaps += 1

        if reset:
            last_end = 0
            reset = False
        else:
            last_end = save_end
        
    E.info( "%s" % str(counter) )

def annotateTTS( iterator, fasta, options ):
    """annotate termination sites within iterator.

    Entries specified with ``--restrict-source are annotated``.
    """

    gene_iterator = GTF.gene_iterator( iterator )

    ngenes, ntranscripts, npromotors = 0, 0, 0

    for gene in gene_iterator:
        ngenes += 1
        is_negative_strand = Genomics.IsNegativeStrand( gene[0][0].strand )
        lcontig = fasta.getLength( gene[0][0].contig )
        tts = []
        transcript_ids = []
        for transcript in gene:

            ntranscripts += 1
            mi, ma = min( [x.start for x in transcript ] ), max( [x.end for x in transcript ] )
            transcript_ids.append( transcript[0].transcript_id )
            # if tts is directly at start/end of contig, the tss will be within an exon.
            # otherwise, it is outside an exon.
            if is_negative_strand:
                #tss.append( (min( lcontig-options.promotor, ma), min(lcontig, ma + options.promotor)) )
                tts.append( (max(0, mi-options.promotor), max(options.promotor,mi)) )
            else:
                #tss.append( (max(0,mi - options.promotor), max(options.promotor,mi)) )
                tts.append( (min(ma,lcontig-options.promotor), min(lcontig,ma + options.promotor)) )

        if options.merge_promotors:
            # merge the promotors (and rename - as sort order might have changed)
            tts = Intervals.combine( tts )
            transcript_ids = ["%i" % (x+1) for x in range(len(tts) )]
            
        gtf = GTF.Entry()
        gtf.fromGTF( gene[0][0], gene[0][0].gene_id, gene[0][0].gene_id )
        gtf.source = "tts"

        x = 0
        for start, end in tts:
            gtf.start, gtf.end = start, end
            gtf.transcript_id = transcript_ids[x]
            options.stdout.write( "%s\n" % str(gtf) )
            npromotors += 1
            x += 1

    if options.loglevel >= 1:
        options.stdlog.write( "# ngenes=%i, ntranscripts=%i, ntss=%i\n" % (ngenes, ntranscripts, npromotors) )

def annotateGenes( iterator, fasta, options ):
    """annotate gene structures

    This method outputs intervals for first/middle/last exon/intron, UTRs and flanking regions.

    This method annotates per transcript. In order to achieve a unique tiling, 
    use only a single transcript per gene and remove any overlap between 
    genes.
    """

    gene_iterator = GTF.gene_iterator( iterator )

    ngenes, ntranscripts, nskipped = 0, 0, 0

    results = []
    increment =  options.increment

    introns_detail = "introns" in options.detail
    exons_detail = "exons" in options.detail

    for gene in gene_iterator:
        ngenes += 1
        is_negative_strand = Genomics.IsNegativeStrand( gene[0][0].strand )
        try:
            lcontig = fasta.getLength( gene[0][0].contig )
        except KeyError:
            nskipped += 1
            continue

        results = []

        for transcript in gene:

            def _add( interval, anno ):
                gtf = GTF.Entry()
                gtf.contig = transcript[0].contig
                gtf.gene_id = transcript[0].gene_id
                gtf.transcript_id = transcript[0].transcript_id
                gtf.strand = transcript[0].strand
                gtf.feature = anno
                gtf.start, gtf.end = interval
                results.append( gtf )

            ntranscripts += 1

            exons = [ (x.start, x.end) for x in transcript if x.feature == "exon" ]
            if len(exons) == 0: nskipped += 1
            
            exons.sort()
            introns = []
            end = exons[0][1]
            for exon in exons[1:]:
                introns.append( (end, exon[0]) )
                end = exon[1]

            # add flank
            start, end = exons[0][0], exons[-1][1]
            upstream, downstream = [], []
            for x in xrange( 0, options.flank, increment ):
                upstream.append( (start - increment, start) )
                start -= increment
                downstream.append( (end, end + increment) )  
                end += increment

            # remove out-of-bounds coordinates
            upstream = [ x for x in upstream if x[0] >= 0]
            downstream = [ x for x in downstream if x[1] <= lcontig ]

            if is_negative_strand:
                exons.reverse()
                introns.reverse()
                upstream, downstream = downstream, upstream
            
            # add exons
            if exons_detail:
                _add( exons[0], "first_exon" )
                if len(exons) > 1:
                    _add( exons[-1], "last_exon" )
                for e in exons[1:-1]:
                    _add( e, "middle_exon" )
            else:
                for e in exons:
                    _add( e, "exon" )

            # add introns
            if introns_detail:
                if len(introns) > 0:
                    _add( introns[0], "first_intron" )
                if len(introns) > 1:
                    _add( introns[-1], "last_intron" )
                for i in introns[1:-1]:
                    _add( i, "middle_intron" )
            else:
                for i in introns:
                    _add( i, "intron" )

            for x, u in enumerate( upstream ):
                _add(u, "upstream_%i" % (increment * (x+1) ) )

            for x, u in enumerate( downstream ):
                _add(u, "downstream_%i" % (increment * (x+1) ) )
                
            results.sort( key = lambda x: x.feature )

        cache = []
        for key, vals in itertools.groupby( results, key = lambda x: x.feature  ):
            v = list(vals)
            intervals = [(x.start, x.end) for x in v ]
            intervals = Intervals.combine( intervals )
            
            for start, end in intervals:
                r = GTF.Entry()
                r.copy( v[0] )
                r.start, r.end = start, end
                cache.append( r )

        cache.sort( key = lambda x: x.start )
        for r in cache:
            options.stdout.write( "%s\n" % str(r))
                
    E.info( "ngenes=%i, ntranscripts=%i, nskipped=%i\n" % (ngenes, ntranscripts, nskipped) )

##------------------------------------------------------------
def main( argv = None ):

    if not argv: argv = sys.argv

    parser = E.OptionParser( version = "%prog version: $Id: gtf2gff.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.add_option("-i", "--ignore-missing", dest="ignore_missing", action="store_true",
                      help="Ignore transcripts on contigs that are not in the genome-file [default=%default]."  )

    parser.add_option("-s", "--restrict-source", dest="restrict_source", type="choice",
                      choices=("protein_coding", "pseudogene", "lncRNA"),
                      help="restrict input by source [default=%default]."  )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices=("full", "genome", "exons", 
                               "promotors", "tts", 
                               "regulons", "tts-regulons", 
                               "genes",
                               "territories", "tss-territories",
                               "great-domains",
                               ),
                      help="method for defining segments [default=%default]."  )

    parser.add_option("-r", "--radius", dest="radius", type="int",
                      help="radius of a territory [default=%default]."  )

    parser.add_option("-f", "--flank", dest="flank", type="int",
                      help="size of the flanking region next to a gene [default=%default]."  )

    parser.add_option( "--increment", dest="increment", type="int",
                       help="size of increment in flank in genestructure annotation [default=%default]."  )

    parser.add_option("-p", "--promotor", dest="promotor", type="int",
                      help="size of a promotor region [default=%default]."  )

    parser.add_option("-u", "--upstream", dest="upstream", type="int",
                      help="size of region upstream of tss [default=%default]."  )

    parser.add_option("-d", "--downstream", dest="downstream", type="int",
                      help="size of region downstream of tss [default=%default]."  )

    parser.add_option( "--detail", dest="detail", type="choice",
                       choices = ("introns+exons", "exons", "introns" ),
                       help="level of detail for gene structure annotation [default=%default]."  )

    parser.add_option( "--merge-promotors", dest="merge_promotors", action="store_true",
                       help="merge promotors [default=%default]."  )

    parser.add_option( "--min-intron-length", dest="min_intron_length", type="int",
                      help="minimum intron length. If the distance between two consecutive exons is smaller, the region will be marked 'unknown' [default=%default]."  )

    parser.add_option("-o", "--sort", dest="sort", action="store_true",
                      help="sort input before processing. Otherwise, the input is assumed "
                      "to be sorted [default=%default]."  )

    parser.set_defaults(
        genome_file = None,
        flank = 1000,
        increment = 1000,
        max_frameshift_length = 4,
        min_intron_length = 30,
        ignore_missing = False,
        restrict_source = None,
        method = "genome",
        radius = 50000,
        promotor = 5000,
        merge_promotors = False,
        upstream = 5000,
        downstream = 5000,
        detail = "exons",
        sort = False,
        )

    (options, args) = E.Start( parser )
    
    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
    else:
        raise ValueError("please specify a --genome-file")

    if not options.restrict_source:
        iterator = GTF.iterator(options.stdin)

    elif options.restrict_source:
        iterator = GTF.iterator_filtered( GTF.iterator(options.stdin), 
                                          source = options.restrict_source )

    # elif options.method in ("promotors", "tts", "regulons"):
    #     iterator = GTF.iterator_filtered( GTF.iterator(options.stdin), source = "protein_coding")
    # else:
    #     iterator = GTF.iterator(options.stdin)

    if options.sort:
        iterator = GTF.iterator_sorted( iterator, sort_order = "position" )

    if options.method == "full" or options.method == "genome":
        segmentor = annotateGenome( iterator, fasta, options )
    elif options.method == "territories":
        segmentor = buildTerritories( iterator, fasta, 'gene', options )
    elif options.method == "tss-territories":
        segmentor = buildTerritories( iterator, fasta, 'tss', options )
    elif options.method == "exons":
        segmentor = annotateExons( iterator, fasta, options )
    elif options.method == "promotors":
        segmentor = annotatePromoters( iterator, fasta, options )
    elif options.method == "regulons":
        segmentor = annotateRegulons( iterator, fasta, True, options )
    elif options.method == "tts-regulons":
        segmentor = annotateRegulons( iterator, fasta, False, options )
    elif options.method == "tts":
        segmentor = annotateTTS( iterator, fasta, options )
    elif options.method == "genes":
        segmentor = annotateGenes( iterator, fasta, options )
    elif options.method == "great-domains":
        segmentor = annotateGREATDomains( iterator, fasta, options )


    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
codonbias_weights2tsv.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python codonbias_weights2tsv.py --help

Type::

   python codonbias_weights2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import os
import optparse

import CGAT.Experiment as E
import CGAT.CSV as CSV
import CGAT.Genomics as Genomics

def WriteHeader( options ):
    if options.is_frequencies:
        cat = "frequency"
    else:
        cat = "weight"

    options.stdout.write("""#
# Changes between preferred codons. The output
# is the preferred codon together with the %s of the unpreferred codon. I.e.:
# speciesA        speciesB      R       CGT     0.646000        CGC     0.587000
# means:
# CGT is preferred in speciesA and has a %s of 0.64 in speciesB.
# CGC is preferred in speciesB and has a %s of 0.59 in speciesA.
species1\tspecies2\taa\tcodon1\tweight1\tcodon2\tweight2\tdifference
""" % (cat, cat, cat))

def WriteOutput( output, options ):
    """sort output."""

    if options.global_sort:
        output.sort()

    for o in output:
        options.stdout.write( "%s\t%s\t%s\t%s\t%f\t%s\t%f\t%5.2f\t%s\n" % o[1])

def WriteChanges( genome1, genome2, changed, options ):
    """write changed preference codons
    """

    output = []
    
    for aa, changes in changed.items():
        ## change will be from -> to
        changes.sort()

        if len(changes) != 2:
            raise "how is more than a change possible?"

        is_preferred1, pref1, codon1 = changes[0]
        is_preferred2, pref2, codon2 = changes[1]

        # check, if thought out correctly
        if is_preferred1 != False and is_preferred2 != True:
            raise "preferred/unpreferred mixed up."

        percent_difference = abs(pref2-pref1) / (pref1 + pref2) * 200.0

        code = "*" * (int(percent_difference) / 10)

        if options.sort == "percent-difference":
            output.append( (-percent_difference, (genome1, genome2, aa, codon1, pref2, codon2, pref1, percent_difference, code ) ) )
        elif options.sort == "aa":
            output.append( (aa, (genome1, genome2, aa, codon1, pref2, codon2, pref1, percent_difference, code ) ) )

        output.sort()
        
    return output

def WriteOverviewWeights( fields, table, options ):

    output = []
    WriteHeader(options)
    for x in range(1, len(fields)-1):
        for y in range(x+1, len(fields)):
            changed = {}

            for c in table:
                codon = c[0]
                w1 = c[x]
                w2 = c[y]
                t1 = w1 == 1.0 and w2 != 1.0
                t2 = w1 != 1.0 and w2 == 1.0
                if t1 or t2:
                    aa = Genomics.MapCodon2AA( codon )
                    if aa not in changed: changed[aa] = []
                    if t1:
                        changed[aa].append( (t1, w2, codon) )
                    else:
                        changed[aa].append( (t1, w1, codon) )

            output += WriteChanged( fields[x], fields[y], changes, options )            

    WriteOutput( output, options)
    
def WriteOverviewFrequencies( fields, table, options ):

    WriteHeader(options)
    output = []
    
    for x in range(1, len(fields)-1):
        for y in range(x+1, len(fields)):
            frequencies = {}

            ## collect frequencies per amino acid
            for c in table:
                codon = c[0]
                f1 = c[x]
                f2 = c[y]
                aa = Genomics.MapCodon2AA( codon )
                if aa not in frequencies: frequencies[aa] = []
                    
                frequencies[aa].append( (codon, f1, f2) )

            changed = {}
            
            ## sort for both genomes, and check if preference has changed
            for aa, codons in frequencies.items():
                codons.sort( lambda x, y: cmp(x[1], y[1]) )
                pref_codon1 = codons[-1]
                codons.sort( lambda x, y: cmp(x[2], y[2]) )                
                pref_codon2 = codons[-1]

                if pref_codon1 == pref_codon2:
                    continue
                else:
                    changed[aa] = [ (True, pref_codon1[2], pref_codon1[0]),
                                    (False, pref_codon2[1], pref_codon2[0]) ]
                    
            output += WriteChanges( fields[x], fields[y], changed, options )            

    WriteOutput( output, options )
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: codonbias_weights2tsv.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option( "--methods", dest="methods", type="string",
                      help="methods to apply.")

    parser.add_option( "--is-frequencies", dest="is_frequencies", action="store_true",
                      help="data is frequencies (default: weights).")

    parser.add_option( "-s", "--sort", dest="sort", type="choice",
                       choices=("percent-difference", "aa"),
                       help="sort order of output table.")

    parser.add_option( "-g", "--global-sort", dest="global_sort", action="store_true",
                       help="globally sort results (otherwise: by species pair).")

    parser.set_defaults( \
       methods = "",
       is_frequencies = False,
       sort = "percent-difference",
       global_sort= False,
       )

    (options, args) = E.Start( parser )
    if options.methods:
        options.methods = options.methods.split(",")

    fields, table = CSV.ReadTable(sys.stdin)

    ## convert weights to floats
    table = CSV.getConvertedTable( table, range( 1, len(fields) ) )

    for method in options.methods:

        if method == "overview":
            if options.is_frequencies:
                WriteOverviewFrequencies( fields, table, options )
            else:
                WriteOverviewWeights( fields, table, options )            
        



#####################################################
# classes and functions for dealing with the output
# from nucmer
#####################################################

import os, sys, re


class Coords(object):
    '''
    class for reading, writing etc the output from 
    show-coords - with nucmer
    '''
    def __init__(self):
        '''
        initialise class
        '''
        self.fields = []

    def __call__(self, coordsfile):
        '''
        geric parsing of the file -  skips evry line until data
        starts. User needs to know columns
        '''
        inputfiles = coordsfile.readline()
        program = coordsfile.readline()
        spacer = coordsfile.readline()
        self.fields = coordsfile.readline().split("\t")
        for line in coordsfile.readlines():
            yield line[:-1].split("\t")


        




#!/bin/env python

'''
cgat_ruffus_profile.py - analyze ruffus logfile
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script examines the ruffus logfile and collates
summary information.

**-t/--time** choice
   Report times either as ``milliseconds`` or ``seconds``.

.. note::

   All times are wall clock times.

'''

import sys
import os
import imp
import cStringIO
import re
import types
import glob
import optparse
import shutil
import datetime
import logging
import collections

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

class Counter(object):
    '''
    
    This class stores the calls per source as calls can be made by
    different sources in any order.
    '''

    def __init__(self):
        self._durations = collections.defaultdict( list )
        self._started = collections.defaultdict( int )
        self._calls = collections.defaultdict( int )

    def add( self, started, dt, source = None ):
        if self._started[source] != 0 and not started:
            self._durations[source].append( dt - self._started[source] )            
            self._started[source] = 0
        elif self._started[source] == 0 and started:
            self._calls[source] += 1
            self._started[source] = dt
        else:
            raise ValueError("""inconsistent time points for %s, has_started=%s, is_started=%s.
Possibly two pipelines have been running concurrently.
""" % (source,
       self._started[source], started))

    def reset( self, source = None):
        '''reset last event.'''
        self._started[source] = None
        self._calls[source] = 0

    def getDuration( self ): 
        if self._durations:
            x = None
            for source, durations in self._durations.iteritems():
                if x == None: x = durations[0] 
                for y in durations[1:]: x += y
            return x
        else:
            return datetime.timedelta()
        
    def getCalls( self ): 
        return sum( self._calls.values() )

    def getRunning( self ):
        '''get numbers of tasks unfinished or still running.'''
        return len( [x for x,y in self._started.iteritems() if y != 0 ] )

    duration = property(getDuration)
    calls = property(getCalls)
    running = property(getRunning)

def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id$", 
                                    usage = globals()["__doc__"])
    
    parser.add_option( "-l", "--logfile", dest="logfile", type="string",
                       help="name of logfile [default=%default]" )

    parser.add_option( "-t", "--time", dest="time", type="choice",
                       choices=("seconds", "milliseconds" ),
                       help="time to show [default=%default]" )

    parser.add_option( "-f", "--filter", dest="filter", type="choice",
                       choices=("unfinished", "running", "completed", "all" ),
                       help="apply filter to output [default=%default]" )

    parser.add_option( "-i", "--ignore-errors", dest="ignore_errors", action="store_true",
                       help="ignore errors [default=%default]" )

    parser.set_defaults( sections = [],
                         logfile = "pipeline.log",
                         filter = "all",
                         time = "seconds" )

    (options, args) = E.Start( parser, argv )

    rx = re.compile("^[0-9]+" )

    if options.sections:
        profile_sections = options.sections
    else:
        profile_sections = ("task", "job" )

    counts = {}
    for section in profile_sections:
        counts[section] = collections.defaultdict( Counter )

    rootpath = os.path.abspath(".")

    infile = IOTools.openFile( options.logfile )

    for line in infile:
        if not rx.match( line ): continue
        data = line[:-1].split()
        if len(data) < 5: continue
        date, time, level, source = data[:4]

        if re.search( "output generated by", line ): 
            E.info( "resetting counts at line=%s" % line[:-1] )
            for section in profile_sections:
                counts[section] = collections.defaultdict( Counter )
            continue

        if not re.match( "task\.", source ): continue

        dt = datetime.datetime.strptime( " ".join( (date, time) ), "%Y-%m-%d %H:%M:%S,%f")

        msg = "".join( data[4:] )
        
        started_task, completed_task, started_job, completed_job = None, None, None, None

        if re.search( "task.log_at_level.\d+Task=(\S+)", msg ):
            checked_task = re.search( "task.log_at_level.\d+Task=(\S+)", msg ).groups()[0]
        elif re.search( "Job=\[(\S+)->(\S+)\]Missingfile[s]*\[(\S+)\]", msg):
            started_infiles, started_job, missing = re.search( "Job=\[(\S+)->(\S+)\]Missingfile[s]*\[(\S+)\]", msg).groups()
        elif re.search( "Taskentersqueue=(\S+)", msg):
            started_task = re.search( "Taskentersqueue=(\S+)", msg).groups()[0]
        elif re.search( "Job=\[(\S+)->(\S+)\]completed", msg ):
            completed_infiles, completed_job = re.search( "Job=\[(\S+)->(\S+)\]completed", msg ).groups()
        elif re.search( "CompletedTask=(\S+)", msg ):
            completed_task = re.search( "CompletedTask=(\S+)", msg ).groups()[0]
        else:
            continue

        try:
            if started_task:
                counts["task"][started_task].add( True, dt, started_task )
            elif completed_task:
                counts["task"][completed_task].add( False, dt, completed_task )
            elif started_job:
                counts["job"][started_job].add( True, dt, started_job )
            elif completed_job:
                counts["job"][completed_job].add( False, dt, completed_job )
            else:
                raise ValueError( "unknown action")
        except ValueError, msg:
            if not options.ignore_errors:
                raise 

    if options.time == "milliseconds":
        f = lambda d: d.seconds + d.microseconds / 1000
    elif options.time == "seconds":
        f = lambda d: d.seconds + d.microseconds / 1000000

    for section in profile_sections:
        options.stdout.write( "\t".join( ("section", "object", "ncalls", "duration", "percall", "running") ) + "\n" )

        running = []
        for objct, c in counts[section].iteritems():
            
            # apply filters
            if options.filter in ("unfinished", "running") and c.running == 0: 
                continue

            d = f(c.duration)
            if c.calls > 0:
                percall = "%6.3f" %( d / float(c.calls))
            else:
                percall = "na"


            options.stdout.write( "\t".join( \
                    (map( str, \
                              (section, objct, 
                               c.calls,
                               d,
                               percall,
                               c.running,
                               )))) + "\n" )
            
            running.extend( [x for x,y in c._started.iteritems() if y != 0 ] )
            
        options.stdout.write( "#//\n\n" )

        if running:        
            options.stdout.write( "# running %ss\n" % section )
            options.stdout.write( "\n".join( map(str, running )) + "\n" )
            options.stdout.write( "#//\n\n" )

    E.Stop()

if __name__ == "__main__":
    sys.exit(main())



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2plot.py - plot genomic data
===============================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script creates plots from genome wide data supplied in :term:`gff` format.

Usage
-----

The data is given in gff files denoting windows. The score field
in the gff file denotes the value of a window.

Multiple tracks can be displayed in the same plot or in several 
plots underneath each other.

Data can be displayed as a histogram with various styles or
as a heat-map.

The layout of the plot is given by a config file. The config
file is arranged in plots that contain one more data tracks.

Here is an example of a track file::

    ################################################
    ## Start of track file
    ################################################
    ## tracks
    [paralogs-ds]
    filename=paralogs_ds_median_score.gff
    style=matrix

    [orthologs-ds]
    filename=orthologs_ds_median_score.gff
    style=matrix

    [variable-ds]
    filename=variable_ds_median_score.gff
    style=matrix                                                                                                                                                                                                               

    [median<intronic>]
    filename=regions_intronic_median_score.gff

    [<intronic>]
    filename=regions_intronic_mean_score.gff

    ## add multi-track tracks
    [median ds]
    tracks=paralogs-ds, orthologs-ds, variable-ds
    text=dS computed from paralog trees using a
            single omega for each ortholog group.
            For paralogs, the distance is between
            C. elegans and C. briggsae. For
            orthologs, the distance is for the
            C. elegans terminal lineage, only.


    [intron-lengths]
    tracks=median<intronic>,<intronic>

    # add vertical lines to the graph
    [vlines]
    filename=boxes.gff
    color=k
    linewidth=3

    # global options for the figure
    [figure]
    # size of a figure in inches
    figsize=12,12

    # create a plot for the legend
    [legend]
    # size of the legend in inches
    figsize=12,12

    ################################################
    ## End of track file
    ################################################

Example::

   python gff2plot.py --help

Type::

   python gff2plot.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse
import types

import ConfigParser
import matplotlib
import pylab
import matplotlib.ticker
import scipy.stats
import numpy

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def formatGenomicCoordinate( value, pos = None ):
    
    decimals = 0
    exps = ['','k','m','g','t']
    suffix = ""

    value = int(value)
    format = "%%.%if%%s%s" % (int(decimals), suffix)

    for exp in xrange(len(exps)-1, -1, -1):

        if value < 1000L**(exp):
            continue
        else:
            return format % (float(value) / 1000L**(exp), exps[exp])
            break
    else:
        return str(value)

def enterParams( o, params ):
    """enter params from object o if they exists.

    Supply a tuple of name, conversion function for non-string
    options.
    """
    r = {}
    for p in params:
        if type(p) == types.TupleType:
            p, f = p
        else:
            f = str
        if hasattr(o, p):
            r[p] = f(getattr(o,p))
    return r

def normalizeValuesByWindows( data, window_size = None ):
    """normalize to constant window size.

    If no window size is given, the smallest window is used. Windows
    smaller than windows-size are ignored.
    """
    
    if not options.window_size:
        window_size = min( map( lambda x: x[1] - x[0], data ) )
    
    new_values = []
    
    for start, end, value in data:
        if end - start < window_size: continue
        start = start - start % window_size
        for z in range( start, end, window_size ):
            new_values.append( (z, value ) )
    
    new_values.sort()
    
    ## interpolate values for the same windows with average
    xvals = []
    yvals = []
    last_x = None
    values = []
    for x, value in new_values:
        
        if last_x != x:
            if last_x != None:
                xvals.append( last_x )
                yvals.append( numpy.mean( values ) )
            values = []

        last_x = x
        values.append( value )

    return xvals, yvals

#-------------------------------------------------------------------
#-------------------------------------------------------------------
#-------------------------------------------------------------------
def addPlot( ax, track, contig, nplotted, 
             nsubplotted = None,
             nsubplots = None,
             y_min = None, 
             y_max = None):
    """add a track to an axes object.
    """

    if contig not in track.mData: return

    ## step function
    ## datapoint is in window average
    xvals = map( lambda x: (x[1] + x[0]) / 2.0, track.mData[contig] )
    yvals = map( lambda x: x[2], track.mData[contig] )
    l  = len(xvals)

    if nsubplots:
        plotnum = nsubplotted
    else:
        plotnum = nplotted

    if track.style == "matrix":

        ## unequal window sizes confuse the image. Therefore, normalize
        ## xvals and yvals to a constant image size
        matrix = pylab.array( yvals )
        matrix.shape = (1, len(yvals) )

        # make sure that the extent of the image and the plot coincide by
        # using extent
        if nsubplots != None:
            y_width = float(y_max - y_min) / nsubplots
            extent = (min(xvals), max(xvals), y_min + y_width * nsubplotted, y_min + y_width * (nsubplotted + 1) )
        else:
            extent= ( min(xvals), max(xvals), min(yvals), max(yvals) )
        ax.imshow( matrix,
                   cmap=track.color_scheme,
                   extent = extent,
                   interpolation="nearest" )

        symbol = options.symbols[plotnum % len(options.symbols)]
        plot = ax.plot( xvals, yvals, symbol, lw=2 )
    else:
        symbol = options.symbols[plotnum % len(options.symbols)]
        plot = ax.plot( xvals, yvals, symbol )
        
    return plot

#-------------------------------------------------------------------
#-------------------------------------------------------------------
#-------------------------------------------------------------------
def plotContig( contig, tracks, options, plot_legend =  False,
                extra_features = None):
    """plot data for contig."""

    if extra_features and "figure" in extra_features:
        figure = pylab.figure(**enterParams( extra_features['figure'], 
                                           ( ("figsize", lambda x: map(int, x.split(","))), 
                                             ("dpi", int), 
                                            "facecolor", 
                                             "edgecolor") ) )
    else:
        figure = pylab.figure()

    if plot_legend:
        if extra_features and "legend" in extra_features:
            legend = pylab.figure(**enterParams( extra_features['legend'], 
                                                 ( ("figsize", lambda x: map(int, x.split(","))), 
                                                   ("dpi", int), 
                                                   "facecolor", 
                                                   "edgecolor") ) )
        else:
            legend = pylab.figure()
        lx = legend.add_axes( (0.1, 0.1, 0.9, 0.9) )
        lx.set_title( "Legend" )
        lx.set_axis_off()
    else:
        legend = None

    axes = []

    ywidth = 0.8 / float(len(tracks) )
    yoffset = 0.1

    axprops = {}
    ayprops = {}

    min_x, max_x = 1000000000, 0

    for track in tracks:
        if track.mData:
            min_x = min( min_x, min( map( lambda x: (x[0]), track.mData[contig] ) ) )
            max_x = max( max_x, max( map( lambda x: (x[1]), track.mData[contig] ) ) )
        
    ## make sure that we use the same view for all axes
    axprops['xlim'] = (min_x, max_x)

    nplotted = 0
    for track in tracks:

        labels, plots = [], []
    
        ax = figure.add_axes( (0.1, track.mYOffset, 0.8, track.mYWidth), **axprops )
        
        if 'sharex' not in axprops:
            ax.xaxis.set_major_formatter(matplotlib.ticker.FuncFormatter(formatGenomicCoordinate) )
            ax.set_xlabel( "Genomic position / Mb" )
            axprops['sharex'] = ax
        else:
            pylab.setp( ax.get_xticklabels(), visible=False)
            
        ax.set_ylabel( track.mTitle, **ayprops)

        if track.mSubTracks:
            
            ## compute maximum extent of y-axis in all of subtracks
            first = True
            for tt in track.mSubTracks:
                if first:
                    min_y = min( map( lambda x: x[2], tt.mData[contig]) )
                    max_y = max( map( lambda x: x[2], tt.mData[contig]) )
                    first = False
                else:
                    min_y = min(min_y, min( map( lambda x: x[2], tt.mData[contig]) ))
                    max_y = max(max_y, max( map( lambda x: x[2], tt.mData[contig]) ))

            nsubplotted = 0
            for tt in track.mSubTracks:
                plot = addPlot( ax, tt, contig, nplotted, 
                                nsubplotted, len(track.mSubTracks),
                                min_y, max_y )
                nsubplotted += 1
                plots.append( plot )
                if hasattr(tt, "legend" ):
                    labels.append( tt.legend )
                else:
                    labels.append( tt.mTitle )
                
        else:
            min_y = min( map( lambda x: x[2], track.mData[contig]) )
            max_y = max( map( lambda x: x[2], track.mData[contig]) )

            if options.global_colours:
                n_for_colour = nplotted
            else:
                n_for_colour = 0

            plot = addPlot( ax, track, contig, n_for_colour )
            plots.append( plot )
            if hasattr( track, "legend" ):
                lables.append( track.legend )
            else:
                labels.append( track.mTitle )

        ## reduce number of ticks by 2
        old_ticks = ax.get_yticks()
        step_size = (old_ticks[1] - old_ticks[0]) * 2
        new_ticks = list(pylab.arange( old_ticks[0], old_ticks[-1], step_size))
        ax.set_yticks( new_ticks )

        if nplotted % 2 == 0:
            ax.yaxis.set_ticks_position("right")
            ax.yaxis.set_label_position("right")
        else:
            ax.yaxis.set_ticks_position("left")
            ax.yaxis.set_label_position("left")

        # deal with extra_features
        if extra_features:
            for key, config in extra_features.items():
                if key == "vlines":
                    if contig not in config.mData: continue
                    lines = []
                    for start, end, value in config.mData[contig]:
                        lines.append( start )
                        lines.append( end )
                    ax.vlines( lines, min_y, max_y, **enterParams( config, ("colour", "linewidth" ) ) )
            
        nplotted += 1

        if legend:
            lx = legend.add_axes( (0.1, track.mYOffset, 0.8, track.mYWidth) )
            pylab.setp( lx.get_xticklabels(), visible=False)
            lx.set_xticks( [] )
            lx.set_yticks( [] ) 
            lx.text( 0.4, 0.5, track.mTitle )
            lx.legend( plots, labels, 'center left' )
            if hasattr( track, "text" ):
                lx.text( 0.6, 0.2, track.text, size="smaller", 
                         clip_on = True )
                
    ax.set_title( contig )
    # has to be set at the end, otherwise re-scaled?
    ax.set_xlim( min_x, max_x )

    return figure, legend

def readData( infile ):
    """read data from infile."""
    dd = {}
    for line in infile:
        if line[0] == "#": continue
        d = line[:-1].split("\t")
        contig, start, end, score = d[0], int(d[3]), int(d[4]), float(d[5])
        # if contig != "I": continue
        if contig not in dd: dd[contig] = []
        dd[contig].append( (start,end,score) )
    return dd

class Track:
    
    def __init__(self, title, 
                 priority = 0,
                 data = None, 
                 subtracks = None,
                 config = None ):
        self.mTitle = title
        self.mData = data
        self.mSubTracks = subtracks
        self.height = 1.0
        self.priority = priority
        self.style = "default"
        self.color_scheme = None

        if config:
            for key, value in config.items( section ):
                setattr( self, key, value )
                
def layoutTracks( tracks ):
    """layout tracks."""

    # combine subtracks - these are ordered by appearance
    for track in tracks.values():
        if track.mSubTracks:
            s = []
            for subtrack in track.mSubTracks:
                s.append( tracks[subtrack] )
                del tracks[subtrack]
            track.mSubTracks = s

    # sort by priority and compute heights
    total_width = sum( map( lambda x: x.height, tracks.values() ) )
    
    # convert to list - this is the order in which the tracks will
    # be output
    tracks = list( tracks.values() )
    tracks.sort( lambda x,y: cmp( x.priority, y.priority) )

    yoffset = 0.1
    ymax_width = 0.8

    for track in tracks:
        track.mYOffset = yoffset
        width = ymax_width * track.height / total_width
        track.mYWidth = width
        yoffset += width

    return tracks

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff2plot.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-f", "--file", dest="filenames", type="string",
                      help="files[s] to take data from,stdin = -."  )
    parser.add_option("", "--symbols", dest="symbols", type="string",
                      help="symbols to use for each histogram [steps|...]."  )
    parser.add_option( "--slide-show", dest="slide_show", type="choice",
                       choices=("first", "all", "sequence" ),
                       help="do a slide show - otherwise, write image to file." )
    parser.add_option("--config", dest="filename_config", type="string",
                      help="filename of track configuration file.")
    parser.add_option("--dpi", dest="dpi", type="int",
                      help="dpi for hardcopy output." )
    parser.add_option("--window-size", dest="window_size", type="int",
                      help="window-size.")
    parser.add_option("--output-pattern", dest="output_pattern_image", type="string",
                      help="output pattern for images. Should contain a '%(contig)s' pattern .")
    parser.add_option("--global-colours", dest="global_colours", action="store_true",
                      help="cycle through colours for all tracks.")

    
    parser.set_defaults(
        filenames = None,
        symbols = "k-,b-,r-,c-,m-,y-,g-",
        output_pattern_image="%(contig)s.png",
        slide_show = None,
        window_size = None,
        filename_config = None,
        dpi = None,
        global_colours = False,
        )


    (options, args) = E.Start( parser )
    options.symbols=options.symbols.split(",")

    #--------------------------------------------------------
    # collect all the data
    # list of data per source and contig
    tracks = {}
    extra_features = {}

    if options.filenames:
        options.filenames = options.filenames.split(",")
    
        if len(args) > 0:
            options.filenames = args

    if options.filenames:

        for filename in options.filenames:

            if filename == "-":
                infile = sys.stdin
            else:
                infile = IOTools.openFile(filename)

            data = readData( infile )

            if filename != "-": infile.close()

            track[filename] =  Track( title = filename, data = data )

    elif options.filename_config:
        ## get track information from config file
        config = ConfigParser.ConfigParser()
        config.read( os.path.expanduser( options.filename_config ) )

        
        # first extract special sections
        for section in config.sections():
            if section == "vlines":
                infile = IOTools.openFile(config.get( section, "filename" ), "r")
                data = readData(infile)
                infile.close()
                extra_features[section] = Track( title = section,
                                                 data = data,
                                                 config = config )
                config.remove_section( section )
            elif section in ("figure", "legend" ):
                extra_features[section] = Track( title = section,
                                                 data = None,
                                                 config = config )
                config.remove_section( section )
        n = 0
        for section in config.sections():
            
            if config.has_option( section, "filename" ):
                infile = IOTools.openFile( config.get( section, "filename"), "r" )
                data = readData( infile )
                infile.close()

                tracks[section] = Track( title = section,
                                         data = data,
                                         priority = n,
                                         config = config )

            elif config.has_option( section, "tracks" ):
                subtracks = config.get( section, "tracks" )
                subtracks = map( lambda x: x.strip(), subtracks.split(",") )

                tracks[section] = Track( title = section, 
                                         data = None,
                                         config = config,
                                         priority = n,
                                         subtracks = subtracks )
            n += 1
            
        
    # compile set of all contigs
    contigs = set()
    for track in tracks.values():
        if track.mData:
            contigs = contigs.union( track.mData.keys() )

    # re-arrange tracks and subtracks
    tracks = layoutTracks( tracks )

    nplots = 0
    figures = []
    legend = None
    for contig in contigs:
        figure, l =  plotContig( contig, tracks, options, 
                                 plot_legend = legend == None,
                                 extra_features = extra_features )
        figures.append( figure )
        if l: legend = l

    if options.slide_show:
        if options.slide_show == "first":
            pylab.show()
        elif options.slide_show == "all":
            pylab.show()
        elif options.slide_show == "sequence":
            pylab.show()
    else:
        
        extra_args = {}
        if options.dpi:
            extra_args['dpi'] = options.dpi
        
        for contig, figure in zip(contigs, figures):
            params = { 'contig' : contig }
            filename = options.output_pattern_image % params
            E.info("# creating image: %s" % filename )
            figure.savefig( os.path.expanduser( filename ), **extra_args )
        if legend:
            params = { 'contig' : "legend" }
            filename = options.output_pattern_image % params
            E.info("creating image: %s" % filename )
            legend.savefig( os.path.expanduser( filename ), **extra_args )

    E.info( "ninput=%i, ncontigs=%i, nplots=%i" % (len(tracks), nplots, len(contigs) ) )

    E.Stop()



################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree2plot.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree2plot.py - plot a tree as postscript (deprecated)
=====================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Plot a tree in postscript format. This script has been replaced
by :mod:`tree2svg`.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess

from types import *
import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools

TEMPLATE = """
\\begindef

    \\name{tree0} % imported from NEXUS file
    \\version{1.0} % version of treegraph file format

    %
    % tree geometry
    %
    \\width{150}  % width in mm
    \\height{250}  % height in mm
    %     \\autolength  % ignore len{} in nodes
    %     \\variable
    
    %
    % layout
    %
    \\roundness{0.2}  % between 0.0 and 1.0
    \\thickness{0.3}  % in mm
    \\margin{0}{0}{0}{0} % left top right bottom, in mm
    \\style{default}{plain}{9.6}
    \\style{r}{plain}{12}
    
    %     \\style{u1}{plain}{9.6}
    %     \\style{d1}{italic}{9.6}
    %     \\style{u2}{bold}{8.4}
    %     \\style{br}{plain}{14.4}
    %     \\rulepos{5}{20}
    % position of rule
    %     \\separator{:}
    %     \\proof
    
    %
    % brackets
    %
    %     \\bracket{0}{n1}{n2}{example text}
    %     \\brace*{1.5}{root}{root}{example text}
    
    \\enddef
"""

class TreeGraphError(Exception):
    pass

def to_string(tree,
              branchlengths="d1",
              support="d2",
              length_format="%6.4f",
              support_format="%1.5f"):
    """Return a paup compatible tree line.

    branchlengths go into u2
    support goes into d2
    """
    def make_info_string(data,terminal=False):
        s = '\\len{%f}\n' % data.branchlength
        
        if branchlengths: # write only branchlengths, ignore support
            s += '\\%s{%s}\n' % (branchlengths, length_format % data.branchlength)
        if support:
            ## no support for terminal nodes
            if not terminal:
                s += '\\%s{%s}\n' % (support, length_format % data.support)            
        return s
    
    def newickize(node):
        """Convert a node tree to a newick tree recursively."""

        if not tree.node(node).succ:    #terminal
            return '\\r{%s}\n%s' % (tree.node(node).data.taxon, make_info_string(tree.node(node).data,terminal=True))
        else:
            return '%s(\n%s\n)\n' % (make_info_string(tree.node(node).data), ',\n'.join(map(newickize,tree.node(node).succ)))
        return subtree

    treeline='%s\n\label{root}\n(\n%s\n)\n' % (
        TEMPLATE,
        ','.join(map(newickize,tree.node(tree.root).succ)))
    
    return treeline 

class TreeGraph:

    mExecutable = "/net/cpp-group/bin/tgf"

    def __init__( self,
                  options="",
                  branchlengths = "u1",
                  support = "d2",
                  loglevel = 0):
        
        self.mOptions = options
        self.mBranchLengths = branchlengths
        self.mSupport = support
        self.mLogLevel = loglevel
        
    def CreateTemporaryFiles( self ):
        """create temporary files."""
        self.mTempDirectory = tempfile.mkdtemp()
        
        self.mFilenameTempInput = self.mTempDirectory + "/input"
        self.mFilenameTempOutput = self.mTempDirectory + "/input.svg"

    def DeleteTemporaryFiles( self ):
        """clean up."""
        os.remove( self.mFilenameTempInput )
        os.remove( self.mFilenameTempOutput )
        os.rmdir( self.mTempDirectory )

    def SetStderr( self, file = None):
        """set file for dumping stderr."""
        self.mStderr= file
        
    def WriteOutput( self, lines, filename_output = None):
        """write output to file.

        If file is not given, lines are written to stdout.
        """
        
        if filename_output:
            outfile = open(filename_output, "w")
        else:
            outfile = sys.stdout
            
        outfile.write( string.join( lines, "") )

        if filename_output:
            outfile.close()

    def ParseResult( self, trace_file = None, information_file = None ):

        result.Read( trace_file, information_file )
        
        return result
    
    def Run( self, tree ):

        self.CreateTemporaryFiles()

        tempfile = open(self.mFilenameTempInput, "w")        
        tempfile.write( to_string(input_tree,
                                  branchlengths = self.mBranchLengths,
                                  support = self.mSupport) )
        tempfile.close()

        if self.mLogLevel >= 2:
            os.system("cat %s" % self.mFilenameTempInput)
        
        statement = string.join( ( self.mExecutable,
                                   "-v",
                                   self.mFilenameTempInput ),
                                 " ")


        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempDirectory,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise TreeGraphError, "Error in calculating svg file\n%s" % err

        d = open(self.mFilenameTempOutput).readlines()

        self.DeleteTemporaryFiles()
        
        return "".join(d)


if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree2plot.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.set_defaults(
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    lines = filter( lambda x: x[0] != "#", sys.stdin.readlines())

    nexus = TreeTools.Newick2Nexus( lines )
    
    input_tree = nexus.trees[0]
    
    treegraph = TreeGraph( support = None, loglevel = options.loglevel)

    print treegraph.Run( input_tree )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
probeset2gene - 
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

From a list of affymetrix probesets, find the
associated genes and transcripts.

A transcript must overlap the mapped location
of a probeset in order to be considered.

Uses the bioconductor mappings.

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import collections

import CGAT.Experiment as E

import CGAT.Expression as Expression
import rpy2
from rpy2.robjects import r as R

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools

def getProbeset2Gene( database ):
    
    '''build map relating a probeset to an ENSEMBL gene_id'''

    prefix = database[:-len(".db")]
    mapping = prefix + "ENSEMBL" 
    R.library( database )

    # map is a Bimap object
    m = R(mapping)
    
    result = R.toTable(m)

    mapping = collections.defaultdict( list )
    for probeset_id, gene_id in zip(result["probe_id"], 
                                    result["ensembl_id"] ):
        mapping[probeset_id].append(gene_id)

    E.info( "obtained %i mappings: probes=%i, genes=%i" % \
                (len(result),
                 len(set(result["probe_id"])),
                 len(set(result["ensembl_id"])) ) )
    return mapping

def getProbeset2Location( database = "hgu133plus2.db" ):
    '''build map with genomic coordinates for each probeset.

    The mapping is not necessarily unique.
    '''
    
    R.library( database )

    prefix = database[:-len(".db")]

    contigs = dict( R(prefix + "CHRLENGTHS" ) )

    # map is a Bimap object
    result2start = R.toTable( R(prefix + "CHRLOC" ) )
    result2end = R.toTable( R(prefix + "CHRLOCEND" ) )

    mapping = collections.defaultdict( list )

    # make sure order is the same
    assert result2start["probe_id"] == result2end["probe_id"]

    for probeset_id, contig, start, end in zip(result2start["probe_id"], 
                                          result2start["Chromosome"],
                                          result2start["start_location"],
                                          result2end["end_location"] ):
        
        if start < 0: 
            start = contigs[contig] - start
            end = contigs[contig] - end

        mapping[probeset_id].append( (contig, start, end) )

    E.info( "mappings: probes=%i, contigs=%i" % \
                (len(set(result2start["probe_id"])),
                 len(set(result2start["Chromosome"])),
                 ))

    return mapping

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-d", "--database", dest="database", type="string",
                      help="bioconductor database to use [default=%default]."  )

    parser.add_option("-m", "--mapping", dest="database", type="string",
                      help="bioconductor mapping to use [default=%default]."  )

    parser.add_option("-g", "--filename-gtf", dest="filename_gtf", type="string",
                      help="filename with the gene set in gtf format [default=%default]."  )


    parser.set_defaults(
        database = "mouse4302.db",
        mapping = "ENSEMBL",
        filename_gtf = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    prefix = options.database[:-len(".db")]

    mapping_probeset2gene = prefix + options.mapping
    mapping_probeset2loc  = prefix + "CHRLOC"

    probeset2gene = getProbeset2Gene( 
        database = options.database,
        )

    probeset2location = getProbeset2Location( 
        database = options.database,
        )

    # gtf = GTF.readAndIndex( 
    #     GTF.iterator( IOTools.openFile( options.filename_gtf ) ) )

    counts = E.Counter()

    outfile_notfound = open("notfound.table", "w" )

    options.stdout.write( "probeset_id\tgene_id\tngenes\n" )

    for probeset, locations in probeset2location.iteritems():
        counts.probesets += 1
        gene_ids = probeset2gene[probeset]
        if len(gene_ids) == 0:
            counts.notfound += 1
            continue

        for gene_id in gene_ids:
            options.stdout.write("%s\t%s\t%i\n" % (probeset, gene_id, len(gene_ids)) )
        counts.output += 1

    E.info( "%s" % str(counts))

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




####
####
##
## Project PythonTools
##
## Copyright (C) 2002 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: IntervallsWeighted.py 2784 2009-09-10 11:41:14Z andreas $
##
##
####
####
"""
IntervallsWeigted.py - working with weigted intervals
=====================================================

Work with weighted invervalls. A weighted intervall is
a tuple of the form (from,to,weight).

Funktions in this module take an optional function parameter object fct,
that will give the weight if two intervalls are combined. The default
is to add the weights of intervalls that are combined.

This module is work in progress. Finished are:

CombineIntervallsLarge
RemoveIntervallsSpanning
"""

#----------------------------------------------------------------
def CombineIntervallsLarge( intervalls, fct = lambda x,y: x+y ):
    """combine intervalls. Overlapping intervalls
    are concatenated into larger intervalls.
    """
    if not intervalls:
        return []

    new_intervalls = []
    
    intervalls.sort()
    first_from, last_to, last_weight = intervalls[0]
    
    for this_from, this_to, this_weight in intervalls[1:]:
        if this_from > last_to:
            new_intervalls.append( (first_from, last_to, last_weight ) )
            first_from, last_to, last_weight = this_from, this_to, this_weight
            continue

        if last_to < this_to:
            last_to = this_to

        last_weight = fct(last_weight, this_weight)
    
    new_intervalls.append( ( first_from, last_to, last_weight ))

    return new_intervalls

#----------------------------------------------------------------
def ComplementIntervalls( intervalls, first = None, last = None):
    """complement a list of intervalls with intervalls not
    in list.
    """

    if not intervalls:
        if first and last:
            return [(first,last)]
        else:
            return []
        
    new_intervalls = []
    
    intervalls.sort()
    last_from, last_to = intervalls[0]

    if first != None and first < last_from:
        new_intervalls.append( (first, last_from-1) )

    for this_from, this_to in intervalls:
        if this_from > last_to+1:
            new_intervalls.append( (last_to+1, this_from-1 ) )            

        last_from = this_from
        last_to = max(last_to, this_to)

    if last and last > last_to+1:
        new_intervalls.append( (last_to+1, last))

    return new_intervalls

#----------------------------------------------------------------
def AddComplementIntervalls( intervalls, first = None, last = None):
    """complement a list of intervalls with intervalls not
    in list and return both.
    """

    return intervalls + ComplementIntervalls( intervalls, first, last)

#----------------------------------------
def CombineIntervallsDistance( intervalls, min_distance ):
    """combine a list of non-overlapping intervalls,
    and merge those that are less than a certain
    distance apart.
    """
    
    if not intervalls: return []

    new_intervalls = []
    
    intervalls.sort()
    
    first_from, last_to = intervalls[0]
    
    for this_from, this_to in intervalls[1:]:
        
        if this_from - last_to - 1 >= min_distance:
            new_intervalls.append( (first_from, last_to ) )
            first_from = this_from

        last_to = this_to
    
    new_intervalls.append( ( first_from, last_to ))

    return new_intervalls

#----------------------------------------
def DeleteSmallIntervalls( intervalls, min_length ):
    """combine a list of non-overlapping intervalls,
    and delete those that are too small.
    """
    
    if not intervalls: return []

    new_intervalls = []

    for this_from, this_to in intervalls:
        if (this_to - this_from + 1) >= min_length:
            new_intervalls.append( (this_from, this_to ) )
            
    return new_intervalls

#----------------------------------------------------------------
def CombineIntervallsOverlap( intervalls ):
    """combine intervalls.
    Overlapping intervalls are reduced to their intersection.

    first_from, last_to contain region of current maximum overlapping segment.
    max_right is maximum extension of any sequence overlapping with current
    overlapping segment.
    
    """
    if not intervalls:
        return []

    new_intervalls = []
    
    intervalls.sort()
    
    biggest_from, smallest_to = intervalls[0]
    biggest_to = smallest_to

    tos = []

    for this_from, this_to in intervalls[1:]:

        # no overlap: write everything and reset
        if this_from > biggest_to:
            
            new_intervalls.append( (biggest_from, smallest_to ) )
##             print "-->", biggest_from, smallest_to
            biggest_from = this_from
            smallest_to = this_to
            biggest_to = this_to
##             print this_from, this_to, "biggest_from=", biggest_from, "smallest_to=", smallest_to, "biggest_to=", biggest_to
            tos = []
            continue

        # no overlap with common region: write and reset overlap to new start
        elif this_from > smallest_to:
            new_intervalls.append( (biggest_from, smallest_to ) )
##             print "-->", biggest_from, smallest_to
            biggest_from = this_from
            tos = filter( lambda x, y = biggest_from: x > y, tos )
            if tos:
                smallest_to = min(tos)
            else:
                smallest_to = biggest_to
            
        biggest_to = max( biggest_to, this_to )
        biggest_from = max( biggest_from, this_from)
        smallest_to = min( smallest_to, this_to)

        tos.append( this_to )
        
##         print this_from, this_to, "biggest_from=", biggest_from, "smallest_to=", smallest_to, "biggest_to=", biggest_to, tos

    new_intervalls.append( ( biggest_from, smallest_to ))

    return new_intervalls

#----------------------------------------------------------------
def RemoveIntervallsContained( intervalls ):
    """
    remove intervalls that are fully contained in another.

    [(10, 100), (20, 50), (70, 120), (130, 200), (10, 50), (140, 210), (150, 200)]

    results:
    
    [(10, 100), (70, 120), (130, 200), (140, 210)]   
    """
    if not intervalls:
        return []

    new_intervalls = []
    
    intervalls.sort()
    last_from, last_to = intervalls[0]
    
    for this_from, this_to in intervalls[1:]:
        # this is larger:
        if this_from <= last_from and this_to >= last_to:
            last_from,last_to = this_from,this_to
            continue

        # last is larger
        if last_from <= this_from and last_to >= this_to:
            continue

        # no complete overlap
        new_intervalls.append( (last_from, last_to ) )

        last_from,last_to = this_from,this_to        
        
    new_intervalls.append( ( last_from, last_to ))
    
    return new_intervalls

#----------------------------------------------------------------
def RemoveIntervallsSpanning( intervalls, fct = lambda x,y: x+y ):
    """remove intervalls that are full covering
    another, i.e. always keep the smallest.

    [(10, 100), (20, 50), (70, 120), (40,80), (130, 200), (10, 50), (140, 210), (150, 200)]

    result:
    
    [(20, 50), (40, 80), (70, 120), (150, 200)]
    """
    
    if not intervalls: return []
    
    intervalls.sort()
    
    last_intervalls = intervalls
    while 1:

        new_intervalls = []

        last_from, last_to, last_weight = last_intervalls[0]
        
        for this_from, this_to, this_weight in last_intervalls[1:]:
            # print last_from, last_to, this_from, this_to
            # this is larger:
            if this_from <= last_from and this_to >= last_to:
                continue
            
            # last is larger:
            if last_from <= this_from and last_to >= this_to:
                last_from,last_to,last_weight = this_from,this_to,this_weight
                continue

            # no complete overlap
            new_intervalls.append( (last_from, last_to, last_weight ) )
            last_from,last_to,last_weight = this_from,this_to,this_weight        
        
        new_intervalls.append( ( last_from, last_to, last_weight ))

        if len(last_intervalls) == len(new_intervalls): break

        last_intervalls = new_intervalls
    
    return new_intervalls

#----------------------------------------------------------------
def ShortenIntervallsOverlap( intervalls, to_remove ):
    """shorten intervalls, so that there is no
    overlap with another set of intervalls.

    assumption: intervalls are not overlapping
    
    """
    if not intervalls:
        return []

    if not to_remove:
        return interalls
    
    new_intervalls = []
    
    intervalls.sort()
    to_remove.sort()

    current_to_remove = 0
    
    for this_from, this_to in intervalls:

        for remove_from, remove_to in to_remove:
            #print this_from, this_to, remove_from, remove_to
            if remove_to < this_from: continue
            if remove_from > this_to: continue

            if remove_from <= this_from and remove_to >= this_to:
                this_from = remove_to
                break

            if this_from < remove_from:
                new_intervalls.append( (this_from, remove_from) )
                # print "adding", this_from, remove_from
            
            this_from = max(this_from, remove_to)

            if this_to < this_from: break
            
        if this_to > this_from:
            # print "adding", this_from, this_to
            new_intervalls.append( (this_from, this_to) )            


    return new_intervalls





"""
CSV.py - Tools for parsing CSV files
========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

"""

import string, re, types
import csv

def ConvertDictionary( d , map = {}):
    """tries to convert values in a dictionary.

    if map contains 'default', a default conversion is enforced.
    For example, to force int for every column but column id,
    supply map = {'default' : "int", "id" : "str" }
    """

    rx_int = re.compile("^\s*[+-]*[0-9]+\s*$")
    rx_float = re.compile( "^[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?$" )

    # pre-process with 'default'
    if "default" in map:
        k = "default"
        if map[k] == "int":
            default = int
        elif map[k] == "float":
            default = float
        elif map[k] == "string":
            default = str
    else:
        default = False

    for k,vv in d.items():

        if vv == None: continue
        v = vv.strip()
        try:
            if k in map:
                if map[k] == "int":
                    d[k] = int(v)
                elif map[k] == "float":
                    d[k] = float(v)
                elif map[k] == "string":
                    pass
                continue
            elif default:
                if v != "":
                    d[k] = default(v)
                else:
                    d[k] = v
                continue
        except TypeError, msg:
            raise TypeError( "conversion in field: %s, %s" % (k,msg))

        try:
            if rx_int.match( v ):
                d[k] = int(v)
            elif rx_float.match( v ):
                d[k] = float(v)
        except TypeError, msg:
            raise TypeError( "expected string or buffer: offending value = '%s' " % str(v))
        except ValueError, msg:
            raise ValueError( "conversion error: %s, %s" % (msg, str(d)))
    return d

def GetMapColumn2Type( rows, ignore_empty = False, get_max_values = False ):
    """map fields to types based on rows.

    Preference is Int to Float to String.

    If get_max_values is set to true, the maximum values for integer
    columns are returned in a dictionary.
    """
    
    headers = rows[0].keys()
    map_column2type = {}
    is_full = {}

    max_values = {}
    
    for row in rows:
        for h in headers:

            if row[h] == "": continue
            
            is_full[h] = True

            if type(row[h]) == types.IntType:
                t = types.IntType
                if h not in max_values:
                    max_values[h] = int(row[h])
                else:
                    max_values[h] = max(h, int(row[h]))
                    
            elif type(row[h]) == types.FloatType:
                t = types.FloatType
            else:
                continue
            
            map_column2type[h] = t
            
    ignored = []
    for h in headers:
        if h not in map_column2type:
            if h in is_full or not ignore_empty:
                map_column2type[h] = types.StringType
            else:
                ignored.append( h )                

    if get_max_values:
        return map_column2type, ignored, max_values
    else:
        return map_column2type, ignored

##-----------------------------------------------------------
class CommentStripper:
    """iterator class for stripping comments from file.
    """
    
    def __init__(self, infile ):
        self.infile = infile

    def __iter__(self):
        return self
    
    def next(self):
        while 1:
            line = self.infile.next()
            if not line:
                raise StopIteration
            if line[0] != "#":
                return line

class DictReader( csv.DictReader ):
    """like csv.DictReader, but skip comments (lines starting with "#").
    """

    def __init__(self, infile, *args, **kwargs ):
        csv.DictReader.__init__( self,
                                 CommentStripper( infile ),
                                 *args, **kwargs )

#IMS - UnicodeDictReader to read Unicode encoded files
#Taken from http://stackoverflow.com/questions/1846135/python-csv-library-with-unicode-utf-8-support-that-just-works?rq=1
# WARNING: Does not strip # comment lines yet

class UnicodeCsvReader(object):
    def __init__(self, f, encoding="utf-8", **kwargs):
        self.csv_reader = csv.reader(f, **kwargs)
        self.encoding = encoding

    def __iter__(self):
        return self

    def next(self):
        # read and split the csv row into fields
        row = self.csv_reader.next() 
        # now decode
        return [unicode(cell, self.encoding) for cell in row]

    @property
    def line_num(self):
        return self.csv_reader.line_num

class UnicodeDictReader(csv.DictReader):
    def __init__(self, f, encoding="utf-8", fieldnames=None, **kwds):
        csv.DictReader.__init__(self, f, fieldnames=fieldnames, **kwds)
        self.reader = UnicodeCsvReader(f, encoding=encoding, **kwds)


class DictReaderLarge:
    """drop-in for csv.DictReader - handles very large fields
    
    Warning - minimal implementation - does not handle dialects
    """

    def __init__(self, infile, fieldnames, *args, **kwargs ):
        self.mFile = infile
        self.mFieldNames = fieldnames
        self.mNFields = len(fieldnames)

    def __iter__(self): 
        return self
    
    def next(self):
        
        line = self.mFile.next()
        if not line: raise StopIteration
        data = line[:-1].split("\t")
        assert len(data) == self.mNFields
        return dict( zip( self.mFieldNames, data ) )

######################################################################################
def ReadTable( lines,
               as_rows = True,
               with_header = True,
               ignore_incomplete = False,
               dialect = "excel-tab" ):
    """read a table from infile

    returns table as rows or as columns.
    If remove_incomplete, incomplete rows are simply ignored.
    """

    if type(lines) == types.FileType:
        lines = lines.readlines()

    lines = filter(lambda x: x[0] != "#", lines)

    if len(lines) == 0: return [], []

    if with_header:
        fields = lines[0][:-1].split("\t")
        del lines[0]
    else:
        fields = lines[0][:-1].split("\t")
        fields = map(str, range(len(fields)))

    nfields = len(fields)
    
    try:
        reader = csv.reader( lines.__iter__(), 
                             dialect = dialect )
    except TypeError:
        reader = csv.reader( lines.__iter__() )

    table = list(reader)
    
    if ignore_incomplete:
        table = [ x for x in table if len(x) == nfields ]
    else:
        for r, row in enumerate(table):
            if len(row) != nfields:
                if not ignore_incomplete:
                    raise ValueError( "missing elements in line %s, received=%s, expected=%s" % \
                                          (r, str(row),  str(fields)) )

                raise ValueError
        
    if not as_rows: table = zip( *table )
        
    return fields, table

######################################################################################
def ReadTables( infile, *args, **kwargs ):
    """read a set of csv tables. 

    Individual tables are separated by // on a single line.
    """
    
    lines = filter(lambda x: x[0] != "#", infile.readlines())
    chunks = filter( lambda x: lines[x][:2] == "//", range( len(lines )) )
    if not lines[-1].startswith("//"): chunks.append( len(lines) )

    class Result: pass
    result = []
    
    start = 0
    for end in chunks:
        
        fields, table = ReadTable( lines[start:end], *args, **kwargs )
        r = Result()
        r.mFields = fields
        r.mTable = table
        result.append(r)
        start = end + 1

    return result

######################################################################################
## group rows in table
def __DoGroup( rows, group_column, group_function, missing_value = "na"):

    values = []
    for x in range(len(rows[0])):
        if x == group_column:
            values.append( rows[0][x] )
        else:
            v = filter( lambda x: x != missing_value, map(lambda y: y[x], rows))
            if len(v) == 0:
                values.append( missing_value)
            else:
                values.append( group_function(map(lambda y: y[x], rows)))

    return values
    
def GroupTable( table,
                group_column = 0,
                group_function = min,
                missing_value = "na"):

    table.sort( lambda x,y: cmp(x[group_column], y[group_column]))

    rows = []
    last_value = None
    new_table = []
    
    for row in table:
        if row[group_column] != last_value:

            if last_value != None:
                new_table.append( __DoGroup( rows, group_column, group_function, missing_value ) )
                
            rows = []
            last_value = row[group_column]
            
        rows.append(row)

    if last_value != None:
        new_table.append( __DoGroup( rows, group_column, group_function, missing_value ) )

    return new_table

def getConvertedTable( table, columns, function = float,
                       skip_errors = False):
                   
    ## convert values to floats (except for group_column)
    ## Delete rows with unconvertable values
    new_table = []
    for row in table:
        skip = False
        for c in columns:
            try:
                row[c] = float(row[c])
            except ValueError, msg:
                if skip_errors:
                    skip = True
                    break
                else:
                    raise ValueError, msg
                
        if not skip: new_table.append(row)
        
    return new_table





################################################################################
#   Gene prediction pipeline 
#
#   $Id: gff2fasta.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2fasta.py - output sequences from genomic features
=====================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script outputs the genomic sequences for intervals within
a :term:`gff` formatted file.

The ouput can be optionall masked and filtered.

Usage
-----

Example::

   python gff2fasta.py --genome-file=hg19 < features.gff > features.fasta

Type::

   python gff2fasta.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.Genomics as Genomics
import CGAT.AGP as AGP
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Intervals as Intervals

import bx.intervals.io
import bx.intervals.intersection

##------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff2fasta.py 2861 2010-02-23 17:36:32Z andreas $")

    parser.add_option( "--is-gtf", dest="is_gtf", action="store_true",
                      help="input is gtf instead of gff."  )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.add_option("-m", "--merge", dest="merge", action="store_true",
                      help="merge adjacent intervals with the same attributes. "
                      "[default=%default]")

    parser.add_option("-e", "--feature", dest="feature", type = "string",
                      help="filter by a feature, for example 'exon', 'CDS'. If "
                      "set to the empty string, all entries are output [%default]."  )

    parser.add_option("-f", "--filename-masks", dest="filename_masks", type = "string",
                      metavar = "gff",
                      help="mask sequences with regions given in gff file [%default]."  )

    parser.add_option( "--remove-masked-regions", dest="remove_masked_regions", action="store_true",
                      help="remove regions instead of masking [%default]."  )

    parser.add_option( "--min-length", dest="min_length", type="int",
                       help="set minimum length for sequences output [%default]" )

    parser.add_option( "--max-length", dest="max_length", type="int",
                       help="set maximum length for sequences output [%default]" )

    parser.add_option( "--extend-at", dest="extend_at", type="choice",
                       choices=("none", "3", "5", "both", "3only", "5only" ),
                       help="extend at no end, 3', 5' or both ends. If "
                       "3only or 5only are set, only the added sequence "
                       "is returned [default=%default]" )

    parser.add_option( "--extend-by", dest="extend_by", type="int",
                       help="extend by # bases [default=%default]" )

    parser.set_defaults(
        is_gtf = False,
        genome_file = None,
        merge = False,
        feature = None,
        filename_masks = None,
        remove_masked_regions = False,
        min_length = 0,
        max_length = 0,
        extend_at = None,
        extend_by = 100,
        )

    (options, args) = E.Start( parser )

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
        contigs = fasta.getContigSizes()

    if options.is_gtf:
        iterator = GTF.transcript_iterator( GTF.iterator( sys.stdin ) )
    else:
        gffs = GTF.iterator( sys.stdin )
        if options.merge:
            iterator = GTF.joined_iterator( gffs )
        else:
            iterator =  GTF.chunk_iterator( gffs )

    masks = None
    if options.filename_masks:
        masks = {}
        with open( options.filename_masks, "r") as infile:
            e = GTF.readAsIntervals( GFF.iterator( infile ) )

        # convert intervals to intersectors
        for contig in e.keys():
            intersector = bx.intervals.intersection.Intersecter()
            for start, end in e[contig]:
                intersector.add_interval( bx.intervals.Interval(start,end) )
            masks[contig] = intersector

    ninput, noutput, nmasked, nskipped_masked = 0, 0, 0, 0
    nskipped_length = 0
    nskipped_noexons = 0

    feature = options.feature

    for ichunk in iterator:

        ninput += 1

        if feature:
            chunk = filter( lambda x: x.feature == feature, ichunk )
        else:
            chunk = ichunk

        if len(chunk) == 0:
            nskipped_noexons += 1
            E.info("no features in entry from %s:%i..%i - %s" % (ichunk[0].contig,
                                                                 ichunk[0].start,
                                                                 ichunk[0].end,
                                                                 str(ichunk[0])))
            continue

        contig, strand = chunk[0].contig, chunk[0].strand 
        if options.is_gtf:
            name = chunk[0].transcript_id
        else:
            name = str(chunk[0].mAttributes)

        lcontig = contigs[contig]
        positive = Genomics.IsPositiveStrand( strand )
        intervals = [ (x.start, x.end) for x in chunk ]
        intervals.sort()

        if masks:
            if contig in masks:
                masked_regions = []
                for start, end in intervals:
                    masked_regions += [ (x.start, x.end) for x in masks[contig].find( start, end ) ]
                
                masked_regions = Intervals.combine( masked_regions )
                if len(masked_regions): nmasked += 1

                if options.remove_masked_regions:
                    intervals = Intervals.truncate( intervals, masked_regions )
                else:
                    raise "unimplemented"

                if len(intervals) == 0:
                    nskipped_masked += 1
                    if options.loglevel >= 1:
                        options.stdlog.write( "# skipped because fully masked: %s: regions=%s masks=%s\n" %\
                                                  (name, str([ (x.start, x.end) for x in chunk ]), masked_regions) )
                    continue

        out = intervals

        if options.extend_at:
            if options.extend_at == "5only":
                intervals = [ (max( 0, intervals[0][0] - options.extend_by ), intervals[0][0]) ]            
            elif options.extend_at == "3only":
                intervals = [ (intervals[-1][1], min( lcontig, intervals[-1][1] + options.extend_by)) ]
            else:
                if options.extend_at in ("5", "both"): intervals[0] = (max( 0, intervals[0][0] - options.extend_by ), intervals[0][1])
                if options.extend_at in ("3", "both"): intervals[-1] = (intervals[-1][0], min( lcontig, intervals[-1][1] + options.extend_by))
            
        if not positive:
            intervals = [ (lcontig - x[1], lcontig - x[0]) for x in intervals[::-1] ]
            out.reverse()

        s = [ fasta.getSequence( contig, strand, start, end ) for start,end in intervals ]
        l = sum( [len(x) for x in s ] )
        if l < options.min_length or (options.max_length and l > options.max_length):
            nskipped_length += 1
            if options.loglevel >= 1:
                options.stdlog.write( "# skipped because length out of bounds %s: regions=%s len=%i\n" %\
                                          (name, str(intervals), l) )
            continue

        options.stdout.write( ">%s %s:%s:%s\n%s\n" % (name, 
                                                      contig,
                                                      strand,
                                                      ";".join( ["%i-%i" % x for x in out] ),
                                                      "\n".join( s )) )
        
        noutput += 1

    E.info( "ninput=%i, noutput=%i, nmasked=%i, nskipped_noexons=%i, nskipped_masked=%i, nskipped_length=%i" %\
                (ninput, noutput, nmasked, nskipped_noexons, nskipped_masked, nskipped_length ) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
malis2profiles.py - build profiles from malis
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert a set of plain alignments to profiles.

Usage
-----

Example::

   python malis2profiles.py --help

Type::

   python malis2profiles.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import random
import types

USAGE="""python %s [OPTIONS]



""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.Mali as Mali

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: malis2profiles.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.set_defaults(
        )

    (options, args) = E.Start( parser )
    
    mali = Mali.SequenceCollection()
    last_id = None
    ninput, noutput, nskipped = 0, 0, 0

    for line in sys.stdin:
        if line[0] == "#": continue

        start, ali, end, id = line[:-1].split("\t")
        ninput += 1
        if id != last_id:
            if last_id:
                mali.setName(last_id)
                mali.writeToFile( sys.stdout, format="profile" )
                noutput += 1
            mali = Mali.SequenceCollection()
            last_id = id

        mali.addSequence( id, start, end, ali )
    
    if last_id:
        mali.setName(last_id)
        mali.writeToFile( sys.stdout, format="profile" )
        noutput += 1

    if options.loglevel >= 1:
        options.stdlog.write("# ninput=%i, noutput=%i, nskipped=%i.\n" % (ninput, noutput, nskipped))
        
    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Intervalls.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
def CombineIntervallsLarge( intervalls ):
    """combine intervalls. Overlapping intervalls
    are concatenated into larger intervalls.
    """
    if not intervalls:
        return []

    new_intervalls = []
    
    intervalls.sort()
    first_from, last_to = intervalls[0]
    
    for this_from, this_to in intervalls[1:]:
        if this_from > last_to:
            new_intervalls.append( (first_from, last_to ) )
            first_from, last_to = this_from, this_to
            continue

        if last_to < this_to:
            last_to = this_to
    
    new_intervalls.append( ( first_from, last_to ))

    return new_intervalls

#----------------------------------------------------------------
def ComplementIntervalls( intervalls, first = None, last = None):
    """complement a list of intervalls with intervalls not
    in list.
    """

    if not intervalls:
        if first and last:
            return [(first,last)]
        else:
            return []
        
    new_intervalls = []
    
    intervalls.sort()
    last_from, last_to = intervalls[0]

    if first != None and first < last_from:
        new_intervalls.append( (first, last_from-1) )

    for this_from, this_to in intervalls:
        if this_from > last_to+1:
            new_intervalls.append( (last_to+1, this_from-1 ) )            

        last_from = this_from
        last_to = max(last_to, this_to)

    if last and last > last_to+1:
        new_intervalls.append( (last_to+1, last))

    return new_intervalls

#----------------------------------------------------------------
def AddComplementIntervalls( intervalls, first = None, last = None):
    """complement a list of intervalls with intervalls not
    in list and return both.
    """

    return intervalls + ComplementIntervalls( intervalls, first, last)

#----------------------------------------
def CombineIntervallsDistance( intervalls, min_distance ):
    """combine a list of non-overlapping intervalls,
    and merge those that are less than a certain
    distance apart.
    """
    
    if not intervalls: return []

    new_intervalls = []
    
    intervalls.sort()
    
    first_from, last_to = intervalls[0]
    
    for this_from, this_to in intervalls[1:]:
        
        if this_from - last_to - 1 >= min_distance:
            new_intervalls.append( (first_from, last_to ) )
            first_from = this_from

        last_to = this_to
    
    new_intervalls.append( ( first_from, last_to ))

    return new_intervalls

#----------------------------------------
def DeleteSmallIntervalls( intervalls, min_length ):
    """combine a list of non-overlapping intervalls,
    and delete those that are too small.
    """
    
    if not intervalls: return []

    new_intervalls = []

    for this_from, this_to in intervalls:
        if (this_to - this_from + 1) >= min_length:
            new_intervalls.append( (this_from, this_to ) )
            
    return new_intervalls

#----------------------------------------------------------------
def CombineIntervallsOverlap( intervalls ):
    """combine intervalls.
    Overlapping intervalls are reduced to their intersection.

    first_from, last_to contain region of current maximum overlapping segment.
    max_right is maximum extension of any sequence overlapping with current
    overlapping segment.
    
    """
    if not intervalls:
        return []

    new_intervalls = []
    
    intervalls.sort()
    
    biggest_from, smallest_to = intervalls[0]
    biggest_to = smallest_to

    tos = []

    for this_from, this_to in intervalls[1:]:

        # no overlap: write everything and reset
        if this_from > biggest_to:
            
            new_intervalls.append( (biggest_from, smallest_to ) )
##             print "-->", biggest_from, smallest_to
            biggest_from = this_from
            smallest_to = this_to
            biggest_to = this_to
##             print this_from, this_to, "biggest_from=", biggest_from, "smallest_to=", smallest_to, "biggest_to=", biggest_to
            tos = []
            continue

        # no overlap with common region: write and reset overlap to new start
        elif this_from > smallest_to:
            new_intervalls.append( (biggest_from, smallest_to ) )
##             print "-->", biggest_from, smallest_to
            biggest_from = this_from
            tos = filter( lambda x, y = biggest_from: x > y, tos )
            if tos:
                smallest_to = min(tos)
            else:
                smallest_to = biggest_to
            
        biggest_to = max( biggest_to, this_to )
        biggest_from = max( biggest_from, this_from)
        smallest_to = min( smallest_to, this_to)

        tos.append( this_to )
        
##         print this_from, this_to, "biggest_from=", biggest_from, "smallest_to=", smallest_to, "biggest_to=", biggest_to, tos

    new_intervalls.append( ( biggest_from, smallest_to ))

    return new_intervalls

#----------------------------------------------------------------
def RemoveIntervallsContained( intervalls ):
    """
    remove intervalls that are fully contained in another.

    [(10, 100), (20, 50), (70, 120), (130, 200), (10, 50), (140, 210), (150, 200)]

    results:
    
    [(10, 100), (70, 120), (130, 200), (140, 210)]   
    """
    if not intervalls:
        return []

    new_intervalls = []
    
    intervalls.sort()
    last_from, last_to = intervalls[0]
    
    for this_from, this_to in intervalls[1:]:
        # this is larger:
        if this_from <= last_from and this_to >= last_to:
            last_from,last_to = this_from,this_to
            continue

        # last is larger
        if last_from <= this_from and last_to >= this_to:
            continue

        # no complete overlap
        new_intervalls.append( (last_from, last_to ) )

        last_from,last_to = this_from,this_to        
        
    new_intervalls.append( ( last_from, last_to ))
    
    return new_intervalls

#----------------------------------------------------------------
def RemoveIntervallsSpanning( intervalls ):
    """remove intervalls that are full covering
    another, i.e. always keep the smallest.

    [(10, 100), (20, 50), (70, 120), (40,80), (130, 200), (10, 50), (140, 210), (150, 200)]

    result:
    
    [(20, 50), (40, 80), (70, 120), (150, 200)]
    """
    
    if not intervalls: return []
    
    intervalls.sort()
    
    last_intervalls = intervalls
        
    while 1:

        new_intervalls = []
    
        last_from, last_to = last_intervalls[0]
    
        for this_from, this_to in last_intervalls[1:]:
            # print last_from, last_to, this_from, this_to
            # this is larger:
            if this_from <= last_from and this_to >= last_to:
                continue
            
            # last is larger:
            if last_from <= this_from and last_to >= this_to:
                last_from,last_to = this_from,this_to
                continue

            # no complete overlap
            new_intervalls.append( (last_from, last_to ) )
            last_from,last_to = this_from,this_to        
        
        new_intervalls.append( ( last_from, last_to ))

        if len(last_intervalls) == len(new_intervalls): break

        last_intervalls = new_intervalls
    
    return new_intervalls

#----------------------------------------------------------------
def ShortenIntervallsOverlap( intervalls, to_remove ):
    """shorten intervalls, so that there is no
    overlap with another set of intervalls.

    assumption: intervalls are not overlapping
    
    """
    if not intervalls:
        return []

    if not to_remove:
        return interalls
    
    new_intervalls = []
    
    intervalls.sort()
    to_remove.sort()

    current_to_remove = 0
    
    for this_from, this_to in intervalls:

        for remove_from, remove_to in to_remove:
            #print this_from, this_to, remove_from, remove_to
            if remove_to < this_from: continue
            if remove_from > this_to: continue

            if remove_from <= this_from and remove_to >= this_to:
                this_from = remove_to
                break

            if this_from < remove_from:
                new_intervalls.append( (this_from, remove_from) )
                # print "adding", this_from, remove_from
            
            this_from = max(this_from, remove_to)

            if this_to < this_from: break
            
        if this_to > this_from:
            # print "adding", this_from, this_to
            new_intervalls.append( (this_from, this_to) )            


    return new_intervalls

#----------------------------------------------------------------
def CalculateOverlap( intervalls1, intervalls2 ):
    """calculate overlap between intervalls.
    """

    if not intervalls1 or not intervalls2:
        return 0

    intervalls1.sort()
    intervalls2.sort()

    overlap = 0
    x = 0
    y = 0

    while x < len(intervalls1) and y < len(intervalls2):

        xfrom, xto = intervalls1[x]
        yfrom, yto = intervalls2[y]

        if xto < yfrom:
            x += 1
        elif yto < xfrom:
            y += 1
        else:
            overlap += min( xto, yto ) - max(xfrom, yfrom ) + 1
            
            if xto < yto:
                x += 1
            elif yto < yto:
                y += 1
            else:
                x += 1
                y += 1

    return overlap
        
        
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
split_genome.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python split_genome.py --help

Type::

   python split_genome.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse

import CGAT.Experiment as E

USAGE="""python %s [OPTIONS] [genomic_sequence] [ < genomic sequence]

Version: $Id: split_genome.py 2781 2009-09-10 11:33:14Z andreas $

Split a genomic fasta file into smaller segments.
""" % sys.argv[0]

def Print( outfile, fragments, options ):
    """print fragments to outfile and return overhang."""
    s = "".join(fragments)
    l = min(len(s), options.chunk_size)
    outfile.write( "\n".join( s[x:x+options.width] for x in range(0, l, options.width) ) )
    outfile.write( "\n" )
    outfile.flush()
    return s[l:]
    
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: split_genome.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-c", "--chunk-size", dest="chunk_size",
                      help="size of chunks in nucleotides.", type="int")
    parser.add_option("-o", "--filename-pattern-output", dest="filename_pattern_output",
                      help="filename for output (should contain one '%i').", type="string")

    parser.set_defaults(
        chunk_size=200000,
        filename_pattern_output="%i.fasta",
        width=100,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    nchunk = 0
    chunksize = 0
    pos = 0
    fragments = []
    outfile = None
    
    for line in sys.stdin:

        is_header = line[0] == ">"
        
        if is_header or chunksize > options.chunk_size:

            if outfile:
                rest = Print( outfile, fragments, options)
                chunksize = len(rest)
                pos -= chunksize
                fragments = [rest]
                outfile.close()

                
                
            else:
                fragments = []
                chunksize = 0

            nchunk += 1
                
            outfile=open( options.filename_pattern_output % nchunk, "w" )                

            if is_header:
                description = line[1:-1]
                id = re.split("\s", description)[0]
                pos = 0
                
            outfile.write(">%s|%i|%i %s\n" % (id, nchunk, pos, description) )

            if is_header: continue
            
        s = re.sub("\s", "", line[:-1])
        l = len(s)
        pos += l
        chunksize += l
        fragments.append(s)
        
    if outfile:
        rest = Print( outfile, fragments, options)        
        outfile.close()


            
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_check_transitivity.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python graph_check_transitivity.py --help

Type::

   python graph_check_transitivity.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

""" program $Id: graph_check_transitivity.py 2782 2009-09-10 11:40:29Z andreas $

python graph_check_transitivity < graph.in

check whether all edges in a graph are transitive, i.e.,
for every two edges A->B and B->C check whether A->C exists.

Edges are taken to be undirected.
"""
import CGAT.Experiment as E
import CGAT.Histogram as Histogram

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: graph_check_transitivity.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("--filename-missing", dest="filename_missing", type="string",
                      help="missing entries.")
    parser.add_option("--filename-found", dest="filename_found", type="string",
                      help="found entries.")
    parser.add_option("--report-step1", dest="report_step1", type="int",
                      help="report interval for input.")
    parser.add_option("--report-step2", dest="report_step2", type="int",
                      help="report interval for processing.")
    parser.add_option("--use-subsets", dest="subsets", action="store_true",
                      help="do subset calculation. Third field contains a redundancy code.")
    
    parser.set_defaults(
        filename_missing = None,
        filename_found = None,
        report_step1 = 100000,
        report_step2 = 10000,
        subsets = False,
        )

    (options, args) = E.Start( parser )

    # retrieve data
    vals = {}
    niterations = 0
    ninput = 0

    for line in sys.stdin:
        if line[0] == "#": continue
        
        niterations += 1
        
        if options.loglevel >= 1 and (niterations % options.report_step1 == 0):
            options.stdlog.write( "# input: %i\n" % (niterations))
            options.stdlog.flush()

        v1, v2, w = line[:-1].split("\t")[:3]
        
        if v1 == v2: continue
        
        if v1 not in vals: vals[v1] = []
        if v2 not in vals: vals[v2] = []
        
        if not options.subsets:
            w = ninput
            
        vals[v1].append( (v2, w) )
        vals[v2].append( (v1, w) )
            
        ninput += 1

    ## make everything unique
    for key, v1 in vals.items():
        vals[key] = tuple(set(v1))

    keys = vals.keys()
    keys.sort()
    niterations = 0
    nkeys = len(keys)
    missing = []
    ntotal = 0
    nfound = 0
    counted = {}
    nremoved = 0

    if options.filename_found:
        outfile_found = open(options.filename_found, "w")
        
    for v1 in keys:

        niterations += 1
        
        if options.loglevel >= 1 and (niterations % options.report_step2 == 0):
            options.stdlog.write( "# loop: %i\n" % (niterations))
            options.stdlog.flush()

        for v2, c2 in vals[v1]:

            ## only to half-symmetric test
            
            for v3, c3 in vals[v2]:
                if (c2, c3) in counted:
                    nremoved += 1
                    # print "v1=", v1, "v2=", v2, "v3=", v3, "c2=", c2, "c3=", c3, "removed"                    
                    continue

                ## do not do self-comparisons
                if v1 == v3: continue
                if c2 == c3: continue
                
                counted[(c2,c3)] = True
                ntotal += 1
                if v3 in map(lambda x: x[0], vals[v1]) or v1 in map(lambda x: x[0], vals[v3]):
                    nfound += 1
                    if options.filename_found:
                        outfile_found.write( "\t".join( (v1, v2, v3) ) + "\n" )
                    # print "v1=", v1, "v2=", v2, "v3=", v3, "c2=", c2, "c3=", c3, "found"
                else:
                    missing.append( (v1, v2, v3) )
                    # print "v1=", v1, "v2=", v2, "v3=", v3, "c2=", c2, "c3=", c3, "missing"
        
    nmissing = len(missing)

    options.stdout.write( "number of egdes\t%i\n" % ninput)
    options.stdout.write( "number of vertices\t%i\n" % nkeys)
    options.stdout.write( "number of removed triplets\t%i\n" % nremoved)
    options.stdout.write( "number of tested triplets\t%i\t%6.4f\n" % (ntotal, float(ntotal) / float(ntotal)))
    options.stdout.write( "number of realized triplets\t%i\t%6.4f\n" % (nfound, float(nfound) / float(ntotal)))
    options.stdout.write( "number of incomplete triplets\t%i\t%6.4f\n" % (nmissing, float(nmissing) / float(ntotal)))

    if options.filename_missing:
        outfile = open(options.filename_missing, "w")
        for v1, v2, v3 in missing:
            outfile.write( "\t".join( (v1, v2, v3) ) + "\n")
        outfile.close()

    if options.filename_found:
        outfile_found.close()
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_group_links_by_taxonomy.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python graph_group_links_by_taxonomy.py --help

Type::

   python graph_group_links_by_taxonomy.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import optparse

import CGAT.Experiment as E
import scipy
import scipy.stats

USAGE="""python %s [OPTIONS]

Aggregate Evalues according to taxonomic ranges.

Taxonomic ranges are given by: name:from-to

The default input is a tab-separated table, sorted by query with the following fields:
   1. Query
   2. Taxonomic node_id
   3. Weight (blast E-value, etc.)
   4. Identifier of sequence
"""



def printResult( outfile, id, ranges, values_within, values_without, f, options):
    """print results in one line."""

    outfile.write( id )

    if options.write_id:
        
        ## reporting ids: sort by function, instead of selecting value
        
        for header, a, b in ranges:
            
            v = values_within[header]
            if len( v ) == 0:
                f1 = "na\tna"
            else:
                v.sort()
                if options.function == "max": v.reverse()
                f1 = "%s\t%s" % (v[0][1], options.value_format % v[0][0])

            v = values_without[header]
            if len( v ) == 0:
                f2 = "na\tna"
            else:
                v.sort()
                if options.function == "max": v.reverse()
                f2 = "%s\t%s" % (v[0][1], options.value_format % v[0][0])

            outfile.write( "\t%s\t%s" % (f1, f2) )
        
    else:
    
        for header, a, b in ranges:

            v = values_within[header]            
            if len( v ) == 0:
                f1 = "na"
            else:
                f1 =options.value_format % f(v)

            v = values_without[header]
            if len( v ) == 0:
                f2 = "na"
            else:
                f2 =options.value_format % f(v)

            outfile.write( "\t%s\t%s" % (f1, f2) )
        
    outfile.write( "\n" )

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: graph_group_links_by_taxonomy.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take for calculating histograms." )

    parser.add_option("-r", "--range", dest="aranges", type="string", action="append",
                      help="taxonomic range." )

    parser.add_option("-f", "--function", dest="function", type="choice",
                      choices=("count", "sum", "min", "max" ),
                      help="choice of aggregate function." )

    parser.add_option("-i", "--write-id", dest="write_id", action="store_true",
                      help="report id and value." )

    parser.set_defaults(
        aranges = [],
        function="count",
        value_format = "%6.4f",
        write_id = False,
        )
    
    (options, args) = E.Start( parser )

    if options.write_id:
        if options.function not in ("min", "max"):
            raise "only min and max allowed with --write-id option"
        f = None
    else:
        if options.function == "count":
            f = len
        elif options.function == "min":
            f = min
        elif options.function == "max":        
            f = max
        elif options.function == "mean":
            f = scipy.mean

    options.aranges += args

    ranges = []
    for r in options.aranges:

        a, b = r.split(":")
        first, last = map(int, b.split("-"))

        ranges.append( (a, first, last ) )

    ## print header
    options.stdout.write("id" )
    for header, first, last in ranges:
        if options.write_id:
            options.stdout.write("\t%s_within_id\t%s_within\t%s_without_id\t%s_without" % (header, header, header, header) )
        else:
            options.stdout.write("\t%s_within\t%s_without" % (header, header ) )
    options.stdout.write("\n")
    
    values_within = {}
    values_without = {}
    for header, first, last in ranges:
        values_within[header] = []
        values_without[header] = []

    last_id = None

    for line in sys.stdin:
        
        if line[0] == "#": continue
        
        data = line[:-1].split( "\t" )

        if data[0] != last_id:
            if last_id:
                printResult( options.stdout, last_id, ranges, values_within, values_without, f,
                             options = options )
                
            values_within = {}
            values_without = {}
            
            for header, first, last in ranges:
                values_within[header] = []
                values_without[header] = []
                
            last_id = data[0]
            
        val = float(data[2])

        for header, first, last in ranges:
            within = first <= int(data[1]) <= last

            if options.write_id:
                xval = ( val, data[3] )
            else:
                xval = val
                
            if within:
                values_within[header].append( xval )
            else:
                values_without[header].append( xval )
                
    printResult( options.stdout, last_id, ranges, values_within, values_without, f,
                 options = options )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2011 David Sims
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
==================================================
Create a bed file tiling a genome from a fai file
==================================================

:Author: David Sims
:Release: $Id$
:Date: |today|
:Tags: Python

This program takes one input file: a Samtools genome idex file.
It also takes two input parameters: the window/tile size (bases) and the shift size.
By default the shift size is equal to the window size.
The default window size is 1000.


Usage
=====

   python genome_bed -g <genome.fai> -o <output.bed> -w window size -s shift size


Code
====

"""

# load modules

import logging as L
import CGAT.Experiment as E
import sys
import os
import re
import shutil
import itertools
import math
import glob
import time
import gzip
import collections
import random
import optparse
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Bed as Bed

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: snp2table.py 2861 2010-02-23 17:36:32Z davids $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with Samtools indexed genome [default=%default]."  )
    parser.add_option("-w", "--window-size", dest="window", type="int",
                      help="Window size for tiling [default=%default]."  )
    parser.add_option("-s", "--shift-size", dest="shift", type="int",
                      help="Window shift for tiling [default=%default]."  )

    parser.set_defaults(
        genome_file = None,
        window=1000,
        shift=1000,
        output_file = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ninput, nunchanged, nchanged = 0, 0, 0

    # Open input file
    E.info("Opening input file: %s" % options.genome_file)
    fasta = IndexedFasta.IndexedFasta( options.genome_file )
    contigs = fasta.getContigSizes( with_synonyms = False )

    # Open output file
    bed = options.stdout
    shift = options.shift

    # Loop over input files and convert to soft clipped
    nwindows = 0
    ncontigs = 0
    for contig, stop in contigs.iteritems():
        ncontigs += 1
        i=0
        while (i < stop):
            j = min( i+options.window, stop )
            #bed.write( """%(contig)s\t%(i)i\t%(j)i\t%(contig)s:%(i)i..%(j)i\n""" % locals() )
            bed.write( """%(contig)s\t%(i)i\t%(j)i\n""" % locals() )
            nwindows += 1
            i += shift
        
    # Report statistics
    E.info( "ncontigs=%i, nwindows=%i" % (ncontigs,nwindows) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




#########################################################
# provide utility functions for parsing the output from 
# a metaphlan analysis
#########################################################
import sys, os
import gzip
import collections

class ReadMap(object):
    '''
    Each read is mapped to a 
    taxonomic group
    '''
    def __init__(self):
        self.seq_id = None
        self.kingdom = None
        self.phylum = None
        self.c_lass = None
        self.order = None
        self.family = None
        self.genus = None
        self.species = None

    def read(self, seq_id, kingdom, phylum, c_lass, order, family, genus, species):

        self.seq_id, self.kingdom, self.phylum, self.c_lass, self.order, self.family, self.genus, self.species = seq_id, kingdom, phylum, c_lass, order, family, genus, species
        return self

#-------------------------------
class RelativeAbundance(object):
    '''
    parsing the output from relative
    abundance
    '''
    def __init__(self):
        
        '''
        each entry will have an associated
        * taxonomic level
        * name within the taxonomic group
        * abundance estimate
        '''
        self.taxon_level = None
        self.taxon = None
        self.abundance = 0

    def read(self, taxon_level, taxon, abundance):
        '''
        read the data
        '''
        self.taxon_level, self.taxon, self.abundance = taxon_level, taxon, abundance
        return self

##############################
# iterator for read map output
##############################        
def relative_abundance_iterator(infile):
    '''
    read in rel_ab outfile from metaphlan 
    and return relative abundances for
    each taxonomic group
    '''
    taxons = collections.defaultdict(list)
    abundances = collections.defaultdict(list)
    for line in infile.readlines():
        data = line[:-1].split("\t")
        abundance = data[1]
        entries = data[0].split("|")
        taxon = entries[len(entries)-1]
        taxon_names = taxon.split("__")
        if len(taxon_names) == 1: # unclassified set
            taxons[taxon_names[0]].append(taxon_names[0])
            abundances[taxon_names[0]].append(abundance)
        else:
            taxons[taxon_names[0]].append(taxon_names[1])
            abundances[taxon_names[0]].append(abundance)

    # return the taxonomic group and the specific
    # names within the taxonomic group
    for group in taxons.keys():
        for abundance in zip(taxons[group], abundances[group]):
            if group == "k":
                group = "kingdom"
            elif group == "p":
                group = "phylum"
            elif group == "c":
                group = "class"
            elif group == "o":
                group = "order"
            elif group == "f":
                group = "family"
            elif group == "g":
                group = "genus"
            elif group == "s":
                group = "species"
            else:
                group = group
            yield RelativeAbundance().read(group, abundance[0], abundance[1])

##############################
# iterator for read map output
##############################        
def read_map_iterator(infile):
    '''
    iterate over read_map file
    from metaphlan
    '''
    for line in infile.readlines():
        data = line[:-1].split("\t")
        seq_id = data[0]
        data = data[1].split("|")

        # The data follow a hierarchy from more specific to
        # more general taxonomic classes so can iteratively sort them out
        data = [x.split("__")[1] for x in data]
        if len(data) == 7:
            kingdom, phylum, c_lass, order, family, genus, species  = data[0], data[1], data[2], data[3], data[4], data[5], data[6]    
        elif len(data) == 6:
            kingdom, phylum, c_lass, order, family, genus, species = data[0], data[1], data[2], data[3], data[4], data[5], "unclassified"
        elif len(data) == 5:
            kingdom, phylum, c_lass, order, family, genus, species = data[0], data[1], data[2], data[3], data[4], "unclassified", "unclassified"
        elif len(data) == 4:
            kingdom, phylum, c_lass, order, family, genus, species = data[0], data[1], data[2], data[3], "unclassified", "unclassified", "unclassified"
        elif len(data) == 3:
            kingdom, phylum, c_lass, order, family, genus, species = data[0], data[1], data[2], "unclassified", "unclassified", "unclassified", "unclassified"
        elif len(data) == 2:
            kingdom, phylum, c_lass, order, family, genus, species = data[0], data[1], "unclassified", "unclassified", "unclassified", "unclassified", "unclassified"
        elif len(data) < 2:
            raise ValueError("could not assign taxonomy at the phylum level")
        yield ReadMap().read(seq_id, kingdom, phylum, c_lass, order, family, genus, species)

##############################
# count by taxonomic group
##############################        
class Counter(ReadMap):
    '''
    counter class for taxonomic counts
    '''
    def __init__(self):
        self.counts = collections.defaultdict(int)
        self.total = 0

    def count(self, infile):
        '''
        get counts for taxonomic groups i.e. the total of reads 
        that were assigned to a taxonomic group in that
        particular taxonomic group
        '''
        taxonomies = ["kingdom", "phylum", "class", "order", "family", "genus", "species"]
        for read in read_map_iterator(infile): 
            self.total += 1
            if read.kingdom != "unclassified":
                self.counts["kingdom"] += 1
            if read.phylum != "unclassified":
                self.counts["phylum"] += 1
            if read.c_lass != "unclassified":
                self.counts["class"] += 1
            if read.order != "unclassified":
                self.counts["order"] += 1
            if read.family != "unclassified":
                self.counts["family"] += 1
            if read.genus != "unclassified":
                self.counts["genus"] += 1
            if read.genus != "unclassified":
                self.counts["species"] += 1
        return self.counts

    def total_count(self, infile):
        for read in read_map_iterator(infile): 
            self.total += 1
        return self.total
    def count_kingdom(self, infile):
        counts = self.count(infile)
        return x["kingdom"]
    def count_phylum(self, infile):
        counts = self.count(infile)
        return counts["phylum"]
    def count_class(self, infile):
        counts = self.count(infile)
        return counts["class"]
    def count_order(self, infile):
        counts = self.count(infile)
        return counts["order"]
    def count_family(self, infile):
        counts = self.count(infile)
        return counts["family"]
    def count_genus(self, infile):
        counts = self.count(infile)
        return counts["genus"]
    def count_species(self, infile):
        counts = self.count(infile)
        return counts["species"]
    def count_total_species(self, infile):
        species = set()
        for read in read_map_iterator(infile):
            if read.species.find("unclassified") == -1: continue
            species.add(read.species)
        return len(species)
    def proportion_with_clade_assignment(self, infile, total_reads):
        return float(total_reads)/self.total_count(infile)

###########################################
###########################################
###########################################
        



